@@ -18,6 +18,11 @@ import libvirt
 import difflib
 import logging
 import os, sys
+import virtinst
+
+# Force is_blktap_capable to return a consistent value, so test suite
+# won't change based on the system
+virtinst._util.is_blktap_capable = lambda: False
 
 # Setup logging
 rootLogger = logging.getLogger()
@@ -35,9 +40,12 @@ else:
     debug = False
 
 # Used to ensure consistent SDL xml output
-os.environ["HOME"] = "/testdir"
+os.environ["HOME"] = "/tmp"
 os.environ["DISPLAY"] = ":3.4"
 
+def open_testdriver():
+    return libvirt.open("test:///%s/tests/testdriver.xml" % os.getcwd())
+
 # Register libvirt handler
 def libvirt_callback(ignore, err):
     logging.warn("libvirt errmsg: %s" % err[2])
@@ -51,9 +59,10 @@ def read_file(filename):
 
     return out
 
-def diff_compare(actual_out, filename):
+def diff_compare(actual_out, filename, expect_out=None):
     """Compare passed string output to contents of filename"""
-    expect_out = read_file(filename)
+    if not expect_out:
+        expect_out = read_file(filename)
 
     diff = "".join(difflib.unified_diff(expect_out.splitlines(1),
                                         actual_out.splitlines(1),
