@@ -13,6 +13,10 @@ EXCEPTHOOK='__excepthook__'
 BUILTIN_TYPE="(StoragePool.__init__|randomMAC|Capabilities.guestForOSType|acquireKernel|acquireBootDisk|DistroInstaller.__init__|PXEInstaller.__init__|Guest.list_os_variants|Guest.get_os_type_label|Guest.get_os_variant_label|FullVirtGuest.__init__|VirtualNetworkInterface.__init__|VirtualGraphics.__init__|Installer.__init__|Guest.__init__|LiveCDInstaller.__init__|ParaVirtGuest.__init__|VirtOptionParser.print_help|get_max_vcpus|setupLogging.exception_log|VirtualDisk.__init__|disk.__init__|netdev.__init__)|guest_lookup"
 BTYPE_TYPE="${BUILTIN_TYPE}.*Redefining built-in 'type'"
 
+# Built-in type 'format'
+BUILTIN_FORMAT="(*Pool.__init__|*Volume.__init__|find_input)"
+BTYPE_FORMAT="${BUILTIN_FORMAT}.*Redefining built-in 'format'"
+
 # Following functions are in the public api which have argument names that
 # override builtin 'str'.
 BUILTIN_STR="(xml_escape)"
@@ -30,6 +34,12 @@ USE_OF__EXIT="member _exit"
 # pylint doesn't pick it up
 UNDEF_GETTEXT="Undefined variable '_'"
 
+# Don't complain about 'ucred' or 'selinux' not being available
+UCRED="import 'ucred'"
+SELINUX="import 'selinux'"
+COVERAGE="import 'coverage'"
+OLDSELINUX="'selinux' has no "
+
 # Public api error
 VD_MISMATCHED_ARGS="VirtualDisk.get_xml_config.*Arguments number differs"
 
@@ -37,15 +47,20 @@ VD_MISMATCHED_ARGS="VirtualDisk.get_xml_config.*Arguments number differs"
 URLTEST_ACCESS="TestURLFetch.*Access to a protected member"
 
 # We use some hacks in the test driver to simulate remote libvirt URIs
-TEST_HACKS="TestClone.*protected member _util"
+TEST_HACKS="TestClone.*protected member _util|testQEMUDriverName.*protected member _get_uri|Access to a protected member _util"
 
 # Scattered examples of legitimately unused arguments
-UNUSED_ARGS="(SuseDistro|SolarisDistro).isValidStore.*Unused argument 'progresscb'|.*Installer.prepare.*Unused argument|post_install_check.*Unused argument 'guest'|Guest.__init__.*Unused argument 'type'"
+UNUSED_ARGS="(SuseDistro|SolarisDistro|NetWareDistro).isValidStore.*Unused argument 'progresscb'|.*Installer.prepare.*Unused argument|post_install_check.*Unused argument 'guest'|Guest.__init__.*Unused argument 'type'"
 
 # Outside __init__ checks throw false positives with distutils custom commands
 # tests.storage also invokes false positives using hasattr
 OUTSIDE_INIT="(.*Test.*|.*createPool.*)outside __init__"
 
+# pylint complains about some of the subclass funkiness in chardev classes
+CHAR_SUBCLASS=".*VirtualCharDevice' has no '(source_mode|source_path)' member.*|.*Method '_char_xml' is abstract in class 'VirtualCharDevice'.*"
+
+# Crap spewed by python 2.6
+MAX_RECURSION="maximum recursion depth"
 
 # FIXME: Everything skipped below are all bugs
 
@@ -55,7 +70,7 @@ ACCESS__CONN="Access to a protected member _conn"
 
 # There isn't a clean API way to access this functions from the API, but
 # they provide info that is needed. These need need to be fixed.
-PROT_MEM_BUGS="protected member (_lookup_osdict_key|_OS_TYPES|_prepare_install|_create_devices|_install_disks)"
+PROT_MEM_BUGS="protected member (_lookup_osdict_key|_OS_TYPES|_prepare_install|_create_devices|_add_install_dev)|'virtinst.FullVirtGuest' has no '_OS_TYPES'"
 
 DMSG=""
 addmsg() {
@@ -67,6 +82,12 @@ addchecker() {
     DCHECKERS="${DCHECKERS},$1"
 }
 
+addmsg_support() {
+    if `pylint --list-msgs | grep -q $1` ; then
+        addmsg "$1"
+    fi
+}
+
 # Disabled Messages:
 addmsg "C0103"  # C0103: Name doesn't match some style regex
 addmsg "C0111"  # C0111: No docstring
@@ -82,12 +103,16 @@ addmsg "W0702"  # W0702: No exception type specified
 addmsg "R0201"  # R0201: Method could be a function
 
 # Possibly useful at some point
-#addmsg "W0201"  # W0201: Defined outside __init__
 addmsg "W0403"  # W0403: Relative imports
 addmsg "W0511"  # W0511: FIXME and XXX: messages (useful in the future)
 addmsg "C0322"  # C0322: *Operator not preceded by a space*
 addmsg "C0323"  # C0323: *Operator not followed by a space*
 addmsg "C0324"  # C0324: *Comma not followed by a space*
+addmsg "R0401"  # R0401: Cyclic imports
+
+# Not supported in many pylint versions
+addmsg_support "W6501"  # W6501: Using string formatters in logging message
+                        #        (see help message for info)
 
 # Disabled Checkers:
 addchecker "Design"         # Things like "Too many func arguments",
@@ -100,13 +125,18 @@ AWK=awk
 pylint --ignore=coverage.py, $FILES \
   --reports=n \
   --output-format=colorized \
-  --dummy-variables-rgx="dummy|ignore*" \
+  --dummy-variables-rgx="dummy|ignore*|.*ignore" \
   --disable-msg=${DMSG} \
-  --disable-checker=${DCHECKERS} | \
+  --disable-checker=${DCHECKERS} 2>&1 | \
   egrep -ve "$EXCEPTHOOK" \
         -ve "$BTYPE_TYPE" \
         -ve "$BTYPE_FILE" \
         -ve "$BTYPE_STR" \
+        -ve "$BTYPE_FORMAT" \
+        -ve "$UCRED" \
+        -ve "$SELINUX" \
+        -ve "$COVERAGE" \
+        -ve "$OLDSELINUX" \
         -ve "$USE_OF__EXIT" \
         -ve "$UNDEF_GETTEXT" \
         -ve "$VD_MISMATCHED_ARGS" \
@@ -115,6 +145,8 @@ pylint --ignore=coverage.py, $FILES \
         -ve "$UNUSED_ARGS" \
         -ve "$TEST_HACKS" \
         -ve "$PROT_MEM_BUGS" \
+        -ve "$CHAR_SUBCLASS" \
+        -ve "$MAX_RECURSION" \
         -ve "$OUTSIDE_INIT" | \
   $AWK '\
 # Strip out any "*** Module name" lines if we dont list any errors for them
