@@ -14,12 +14,11 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 # MA 02110-1301 USA.
 
-import libvirt
 import virtinst
 from virtinst import VirtualDisk
 
 # Test helpers
-from storage import createPool, createVol
+import tests
 
 import unittest
 import logging
@@ -40,33 +39,15 @@ import os
 
 # We install several storage pools on the connection to ensure
 # we aren't bumping up against errors in that department.
-logging.debug("\n\nStarting 'validation' storage setup.")
-testconn = libvirt.open("test:///default")
+testconn = tests.open_testdriver()
 testcaps = virtinst.CapabilitiesParser.parse(testconn.getCapabilities())
 
 virtimage = virtinst.ImageParser.parse_file("tests/image-xml/image.xml")
 
-offdskpaths = [ "/dev", ]
-for path in offdskpaths:
-    createPool(testconn, virtinst.Storage.StoragePool.TYPE_DISK,
-               tpath=path, start=False, format="dos")
-
-dirpaths = [ "/var/lib/libvirt/images", "/", "/tmp" ]
-for path in dirpaths:
-    createPool(testconn, virtinst.Storage.StoragePool.TYPE_DIR,
-               tpath=path, start=True)
-
-dskpaths = [ "/somedev", "/dev/disk/by-uuid" ]
-for path in dskpaths:
-    createPool(testconn, virtinst.Storage.StoragePool.TYPE_DISK,
-               format="dos", tpath=path, start=False)
-
-# Create a usable pool/vol pairs
-p = createPool(testconn, virtinst.Storage.StoragePool.TYPE_DIR,
-               tpath="/pool-exist", start=True, poolname="pool-exist")
-dirvol = createVol(p, "vol-exist")
-volinst = virtinst.Storage.StorageVolume(pool=p, name="somevol", capacity=1)
-logging.debug("Ending 'validation' storage setup.\n\n")
+volinst = virtinst.Storage.StorageVolume(conn=testconn,
+                                         pool_name="default-pool",
+                                         name="val-vol",
+                                         capacity=1)
 
 args = {
 
@@ -119,47 +100,49 @@ args = {
 },
 
 'disk' : {
-    'init_conns' : [ testconn, None ],
-    '__init__' : {
-        'invalid' : [{ 'path' : 0},
-                     { 'path' : '/root' },
-                     { 'path' : 'valid', 'size' : None },
-                     { 'path' : "valid", 'size' : 'invalid' },
-                     { 'path' : 'valid', 'size' : -1},
-                     { 'path' : 'notblock', 'type' : VirtualDisk.TYPE_BLOCK},
-                     { 'path' :'/dev/null', 'type' : VirtualDisk.TYPE_BLOCK},
-                     { 'path' : None },
-                     { 'path' : "noexist", 'size' : 900000, 'sparse' : False },
-                     { 'path' : "noexist", 'type' : VirtualDisk.DEVICE_CDROM},
-                     { 'volName' : ("pool-exist", "vol-exist")},
-                     { 'conn' : testconn, 'volName' : ("pool-noexist",
-                                                       "vol-exist")},
-                     { 'conn' : testconn, 'volName' : ("pool-exist",
-                                                       "vol-noexist")},
-                     { 'conn' : testconn, 'volName' : ( 1234,
-                                                       "vol-noexist")},
-                     { 'path' : 'valid', 'size' : 1,
-                       'driverCache' : 'invalid' },
-                    ],
+  'init_conns' : [ testconn, None ],
+  '__init__' : {
 
-        'valid' :   [{ 'path' : '/dev/loop0' },
-                     { 'path' : 'nonexist', 'size' : 1 },
-                     { 'path' :'/dev/null'},
-                     { 'path' : None, 'device' : VirtualDisk.DEVICE_CDROM},
-                     { 'path' : None, 'device' : VirtualDisk.DEVICE_FLOPPY},
-                     { 'conn' : testconn, 'volName' : ("pool-exist",
-                                                       "vol-exist")},
-                     { 'conn' : testconn, 'path' : "/pool-exist/vol-exist" },
-                     { 'conn' : testconn, 'path' : "/pool-exist/vol-noexist",
-                       'size' : 1 },
-                     { 'conn' : testconn, 'volInstall': volinst},
-                     { 'path' : 'nonexist', 'size' : 1,
-                       'driverCache' : 'writethrough' },
-                    ]
-                },
-    'shareable' : {
-        'invalid': [ None, 1234 ],
-        'valid': [ True, False ] },
+   'invalid' : [
+    {'path' : 0},
+    { 'path' : '/root' },
+    { 'path' : 'valid', 'size' : None },
+    { 'path' : "valid", 'size' : 'invalid' },
+    { 'path' : 'valid', 'size' : -1},
+    { 'path' : 'notblock', 'type' : VirtualDisk.TYPE_BLOCK},
+    { 'path' :'/dev/null', 'type' : VirtualDisk.TYPE_BLOCK},
+    { 'path' : None },
+    { 'path' : "noexist", 'size' : 900000, 'sparse' : False },
+    { 'path' : "noexist", 'type' : VirtualDisk.DEVICE_CDROM},
+    { 'volName' : ("default-pool", "default-vol")},
+    { 'conn' : testconn, 'volName' : ("pool-noexist", "default-vol")},
+    { 'conn' : testconn, 'volName' : ("default-pool", "vol-noexist")},
+    { 'conn' : testconn, 'volName' : ( 1234, "vol-noexist")},
+    { 'path' : 'valid', 'size' : 1, 'driverCache' : 'invalid' },
+    { 'conn' : testconn, "path" : "/full-pool/newvol.img", "size" : 1,
+      'sparse' : False },
+   ],
+
+   'valid' : [
+    { 'path' : '/dev/loop0' },
+    { 'path' : 'nonexist', 'size' : 1 },
+    { 'path' :'/dev/null'},
+    { 'path' : None, 'device' : VirtualDisk.DEVICE_CDROM},
+    { 'path' : None, 'device' : VirtualDisk.DEVICE_FLOPPY},
+    { 'conn' : testconn, 'volName' : ("default-pool", "default-vol")},
+    { 'conn' : testconn, 'path' : "/default-pool/default-vol" },
+    { 'conn' : testconn, 'path' : "/default-pool/vol-noexist", 'size' : 1 },
+    { 'conn' : testconn, 'volInstall': volinst},
+    { 'path' : 'nonexist', 'size' : 1, 'driverCache' : 'writethrough' },
+    # Full pool, but we are nonsparse
+    { 'conn' : testconn, "path" : "/full-pool/newvol.img", "size" : 1 },
+   ]
+  },
+
+  'shareable' : {
+    'invalid': [ None, 1234 ],
+    'valid': [ True, False ]
+  },
 },
 
 'installer' : {
@@ -183,11 +166,12 @@ args = {
     'init_conns' : [ testconn, None ],
     'location'  : {
         'invalid' : ['nogood', 'http:/nogood', [], None,
-                     ("pool-noexist", "vol-exist"),
-                     ("pool-exist", "vol-noexist"),
+                     ("pool-noexist", "default-vol"),
+                     ("default-pool", "vol-noexist"),
                     ],
         'valid'   : ['/dev/null', 'http://web', 'ftp://ftp', 'nfs:nfsserv',
-                     ("pool-exist", "vol-exist"),
+                     '/tmp', # For installing from local dir tree
+                     ("default-pool", "default-vol"),
                     ]}
 },
 
@@ -195,10 +179,10 @@ args = {
     'init_conns' : [ testconn, None ],
     'location'  : {
         'invalid' : ['path-noexist',
-                     ("pool-noexist", "vol-exist"),
-                     ("pool-exist", "vol-noexist"),
+                     ("pool-noexist", "default-vol"),
+                     ("default-pool", "vol-noexist"),
                     ],
-        'valid'   : ['/dev/null', ("pool-exist", "vol-exist"),
+        'valid'   : ['/dev/null', ("default-pool", "default-vol"),
                     ]}
 },
 
@@ -233,11 +217,47 @@ args = {
                     'valid'   : ['12345678123456781234567812345678']},
     'clone_mac' : { 'invalid' : ['badformat'],
                     'valid'   : ['AA:BB:CC:DD:EE:FF']},
-    'clone_bs'  : { 'invalid' : [], 'valid'   : ['valid']}
+    'clone_bs'  : { 'invalid' : [], 'valid'   : ['valid']},
 },
 
+'inputdev' : {
+    'init_conns' : [ testconn ],
+    'type' : {
+        'valid'   : [ "mouse", "tablet"],
+        'invalid' : [ "foobar", 1234]},
+    'bus' : {
+        'valid'   : [ "ps2", "xen", "usb"],
+        'invalid' : [ "foobar", 1234]},
+},
+
+'chardev' : {
+    'init_conns' : [ testconn ],
+    'source_path': {
+        'invalid'   : [],
+        'valid'     : [ "/some/path" ]},
+    'source_mode': {
+        'invalid'   : [ None ],
+        'valid'     : virtinst.VirtualCharDevice.char_modes },
+    'source_host': {
+        'invalid'   : [],
+        'valid'     : [ "some.source.host" ]},
+    'source_port': {
+        'invalid'   : [ "foobar"],
+        'valid'     : [ 1234 ]},
+    'connect_host': {
+        'invalid'   : [],
+        'valid'     : [ "some.connect.com" ]},
+    'connect_port': {
+        'invalid'   : [ "foobar"],
+        'valid'     : [ 1234 ]},
+    'protocol': {
+        'invalid'   : [ None ],
+        'valid'     : virtinst.VirtualCharDevice.char_protocols },
+}
+
 } # End of validation dict
 
+
 class TestValidation(unittest.TestCase):
 
     def _getInitConns(self, label):
@@ -296,12 +316,15 @@ class TestValidation(unittest.TestCase):
                    (name, paramname, paramvalue))
             raise AssertionError, msg
 
-    def _testArgs(self, obj, testclass, name, exception_check=None):
+    def _testArgs(self, obj, testclass, name, exception_check=None,
+                  manual_dict=None):
         """@obj Object to test parameters against
            @testclass Full class to test initialization against
            @name String name indexing args"""
         logging.debug("Testing '%s'" % name)
         testdict = args[name]
+        if manual_dict != None:
+            testdict = manual_dict
 
         for paramname in testdict.keys():
             if paramname == "init_conns":
@@ -406,6 +429,29 @@ class TestValidation(unittest.TestCase):
             cman = virtinst.CloneManager.CloneDesign(conn)
             self._testArgs(cman, virtinst.CloneManager.CloneDesign, label)
 
+    def testInputDev(self):
+        label = 'inputdev'
+        for conn in self._getInitConns(label):
+            cman = virtinst.VirtualInputDevice(conn)
+            self._testArgs(cman, virtinst.VirtualInputDevice, label)
+
+    def testCharDev(self):
+        label = 'chardev'
+        paramdict = args[label]
+        devs = []
+
+        for conn in self._getInitConns(label):
+            for dev in virtinst.VirtualCharDevice.dev_types:
+                for char in virtinst.VirtualCharDevice.char_types:
+                    devs.append(virtinst.VirtualCharDevice.get_dev_instance(conn, dev, char))
+
+        for dev in devs:
+            custom_dict = {}
+            for key in paramdict.keys():
+                if hasattr(dev, key):
+                    custom_dict[key] = paramdict[key]
+            self._testArgs(dev, virtinst.VirtualCharDevice, label,
+                           manual_dict=custom_dict)
 
 if __name__ == "__main__":
     unittest.main()
