@@ -18,7 +18,7 @@
 # MA 02110-1301 USA.
 #
 
-import popen2
+import subprocess
 import shutil
 import errno
 import sys
@@ -78,10 +78,11 @@ def run_cmd(cmd):
     """
     Return the exit status and output to stdout and stderr.
     """
-    proc = popen2.Popen3(cmd, capturestderr=True)
-    proc.tochild.close()
+    proc = subprocess.Popen(cmd, stderr=subprocess.PIPE,
+                            stdout=subprocess.PIPE,
+                            close_fds=True)
     ret = proc.wait()
-    return ret, proc.fromchild.readlines(), proc.childerr.readlines()
+    return ret, proc.stdout.readlines(), proc.stderr.readlines()
 
 def run_vdiskadm(args):
     """Run vdiskadm, returning the output."""
@@ -98,8 +99,8 @@ def run_vdiskadm(args):
 class disk(object):
     """Definition of an individual disk instance."""
 
-    def __init__(self, path = None, format = None, bus = "ide",
-        type = DISK_TYPE_DISK):
+    def __init__(self, path = None, format = DISK_FORMAT_NONE, bus = "ide",
+                 type = DISK_TYPE_DISK):
         self.path = path
         self.format = format
         self.bus = bus
@@ -128,8 +129,11 @@ class disk(object):
 
     def out_file(self, out_format):
         """Return the relative path of the output file."""
+        if not out_format:
+            return self.path
+
         relout = self.path.replace(disk_suffixes[self.format],
-            disk_suffixes[out_format])
+                                   disk_suffixes[out_format])
         return re.sub(r'\s', '_', relout)
 
     def vdisk_convert(self, absin, absout):
@@ -151,12 +155,17 @@ class disk(object):
         Use qemu-img to convert the given disk.  Note that at least some
         version of qemu-img cannot handle multi-file VMDKs, so this can
         easily go wrong.
+        Gentoo, Debian, and Ubuntu (potentially others) install kvm-img
+        with kvm and qemu-img with qemu. Both would work.
         """
 
         self.clean += [ absout ]
 
         ret, ignore, stderr = run_cmd(["qemu-img", "convert", "-O",
             qemu_formats[out_format], absin, absout])
+        if ret == 127:
+            ret, ignore, stderr = run_cmd(["kvm-img", "convert", "-O",
+                qemu_formats[out_format], absin, absout])
         if ret != 0:
             raise RuntimeError("Disk conversion failed with "
                 "exit status %d: %s" % (ret, "".join(stderr)))
