@@ -19,8 +19,8 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 # MA 02110-1301 USA.
 
-import libxml2
 from virtinst import _virtinst as _
+import _util
 
 class CapabilitiesParserException(Exception):
     def __init__(self, msg):
@@ -93,6 +93,7 @@ class Host(object):
 
         self.features = CapabilityFeatures()
         self.topology = None
+        self.secmodel = None
 
         if not node is None:
             self.parseXML(node)
@@ -103,6 +104,9 @@ class Host(object):
             if child.name == "topology":
                 self.topology = Topology(child)
 
+            if child.name == "secmodel":
+                self.secmodel = SecurityModel(child)
+
             if child.name != "cpu":
                 child = child.next
                 continue
@@ -252,6 +256,21 @@ class TopologyCPU(object):
         self.id = int(node.prop("id"))
 
 
+class SecurityModel(object):
+    def __init__(self, node = None):
+        self.model = None
+        self.doi = None
+
+        if not node is None:
+            self.parseXML(node)
+
+    def parseXML(self, node):
+        for child in node.children or []:
+            if child.name == "model":
+                self.model = child.content
+            elif child.name == "doi":
+                self.doi = child.content
+
 class Capabilities(object):
     def __init__(self, node = None):
         self.host = None
@@ -320,35 +339,9 @@ class Capabilities(object):
             self.host.topology = self._topology
 
 def parse(xml):
-    class ErrorHandler:
-        def __init__(self):
-            self.msg = ""
-        def handler(self, ignore, s):
-            self.msg += s
-    error = ErrorHandler()
-    libxml2.registerErrorHandler(error.handler, None)
-
-    try:
-        # try/except/finally is only available in python-2.5
-        try:
-            doc = libxml2.readMemory(xml, len(xml),
-                                     None, None,
-                                     libxml2.XML_PARSE_NOBLANKS)
-        except (libxml2.parserError, libxml2.treeError), e:
-            raise CapabilitiesParserException("%s\n%s" % (e, error.msg))
-    finally:
-        libxml2.registerErrorHandler(None, None)
-
-    try:
-        root = doc.getRootElement()
-        if root.name != "capabilities":
-            raise CapabilitiesParserException("Root element is not 'capabilities'")
-
-        capabilities = Capabilities(root)
-    finally:
-        doc.freeDoc()
-
-    return capabilities
+    return _util.parse_node_helper(xml, "capabilities",
+                                   Capabilities,
+                                   CapabilitiesParserException)
 
 def guest_lookup(conn, caps=None, os_type=None, arch=None, type=None,
                  accelerated=False):
