@@ -27,12 +27,41 @@ import _util
 import libvirt
 import CapabilitiesParser
 import VirtualGraphics
+from VirtualDevice import VirtualDevice
+from VirtualDisk import VirtualDisk
 
 import osdict
 from virtinst import _virtinst as _
 import logging
 import signal
 
+def _validate_cpuset(conn, val):
+    if val is None or val == "":
+        return
+
+    if type(val) is not type("string") or len(val) == 0:
+        raise ValueError, _("cpuset must be string")
+    if re.match("^[0-9,-]*$", val) is None:
+        raise ValueError, _("cpuset can only contain numeric, ',', or "
+                            "'-' characters")
+
+    pcpus = _util.get_phy_cpus(conn)
+    for c in val.split(','):
+        if c.find('-') != -1:
+            (x, y) = c.split('-')
+            if int(x) > int(y):
+                raise ValueError, _("cpuset contains invalid format.")
+            if int(x) >= pcpus or int(y) >= pcpus:
+                raise ValueError, _("cpuset's pCPU numbers must be less "
+                                    "than pCPUs.")
+        else:
+            if len(c) == 0:
+                continue
+
+            if int(c) >= pcpus:
+                raise ValueError, _("cpuset's pCPU numbers must be less "
+                                    "than pCPUs.")
+    return
 
 class Guest(object):
 
@@ -56,6 +85,26 @@ class Guest(object):
         return Guest._OS_TYPES[type]["variants"][variant]["label"]
     get_os_variant_label = staticmethod(get_os_variant_label)
 
+    def cpuset_str_to_tuple(conn, cpuset):
+        _validate_cpuset(conn, cpuset)
+        pinlist = [False] * _util.get_phy_cpus(conn)
+
+        entries = cpuset.split(",")
+        for e in entries:
+            series = e.split("-", 1)
+
+            if len(series) == 1:
+                pinlist[int(series[0])] = True
+                continue
+
+            start = int(series[0])
+            end = int(series[1])
+
+            for i in range(start, end):
+                pinlist[i] = True
+
+        return tuple(pinlist)
+    cpuset_str_to_tuple = staticmethod(cpuset_str_to_tuple)
 
     def __init__(self, type=None, connection=None, hypervisorURI=None,
                  installer=None):
@@ -71,19 +120,23 @@ class Guest(object):
         self._cpuset = None
         self._graphics_dev = None
         self._consolechild = None
+        self._os_autodetect = False
 
         self._os_type = None
         self._os_variant = None
 
-        # Public device lists unaltered by install process
+        # DEPRECATED: Public device lists unaltered by install process
         self.disks = []
         self.nics = []
         self.sound_devs = []
         self.hostdevs = []
 
-        # Device lists to use/alter during install process
-        self._install_disks = []
-        self._install_nics = []
+        # General device list. Only access through API calls (even internally)
+        self._devices = []
+
+        # Device list to use/alter during install process. Don't access
+        # directly, use internal APIs
+        self._install_devices = []
 
         # The libvirt virDomain object we 'Create'
         self.domain = None
@@ -91,6 +144,9 @@ class Guest(object):
         # Default disk target prefix ('hd' or 'xvd'). Set in subclass
         self.disknode = None
 
+        # Default bus for disks (set in subclass)
+        self._diskbus = None
+
         self.conn = connection
         if self.conn == None:
             logging.debug("No conn passed to Guest, opening URI '%s'" % \
@@ -172,25 +228,11 @@ class Guest(object):
     def get_cpuset(self):
         return self._cpuset
     def set_cpuset(self, val):
-        if type(val) is not type("string") or len(val) == 0:
-            raise ValueError, _("cpuset must be string")
-        if re.match("^[0-9,-]*$", val) is None:
-            raise ValueError, _("cpuset can only contain numeric, ',', or "
-                                "'-' characters")
+        if val is None or val == "":
+            self._cpuset = None
+            return
 
-        pcpus = _util.get_phy_cpus(self.conn)
-        for c in val.split(','):
-            if c.find('-') != -1:
-                (x, y) = c.split('-')
-                if int(x) > int(y):
-                    raise ValueError, _("cpuset contains invalid format.")
-                if int(x) >= pcpus or int(y) >= pcpus:
-                    raise ValueError, _("cpuset's pCPU numbers must be less "
-                                        "than pCPUs.")
-            else:
-                if int(c) >= pcpus:
-                    raise ValueError, _("cpuset's pCPU numbers must be less "
-                                        "than pCPUs.")
+        _validate_cpuset(self.conn, val)
         self._cpuset = val
     cpuset = property(get_cpuset, set_cpuset)
 
@@ -235,7 +277,7 @@ class Guest(object):
             if self._OS_TYPES[self.os_type]["variants"].has_key(val):
                 self._os_variant = val
             else:
-                raise ValueError, _("OS variant '%(var)s; does not exist in "
+                raise ValueError, _("OS variant '%(var)s' does not exist in "
                                     "our dictionary for OS type '%(ty)s'" ) % \
                                     {'var' : val, 'ty' : self._os_type}
         else:
@@ -250,6 +292,12 @@ class Guest(object):
             raise ValueError, _("Unknown OS variant '%s'" % val)
     os_variant = property(get_os_variant, set_os_variant)
 
+    def set_os_autodetect(self, val):
+        self._os_autodetect = bool(val)
+    def get_os_autodetect(self):
+        return self._os_autodetect
+    os_autodetect = property(get_os_autodetect, set_os_autodetect)
+
     # Get the current variants 'distro' tag: 'rhel', 'fedora', etc.
     def get_os_distro(self):
         return self._lookup_osdict_key("distro")
@@ -383,57 +431,134 @@ class Guest(object):
     # END DEPRECATED PROPERTIES
 
 
+    # Device Add/Remove Public API methods
+
+    def _dev_build_list(self, devtype, devlist=None):
+        if not devlist:
+            devlist = self._devices
+
+        newlist = []
+        for i in devlist:
+            if i.virtual_device_type == devtype:
+                newlist.append(i)
+        return newlist
+
+    def add_device(self, dev):
+        """
+        Add the passed device to the guest's device list.
+
+        @param dev: VirtualDevice instance to attach to guest
+        """
+        if not isinstance(dev, VirtualDevice):
+            raise ValueError(_("Must pass a VirtualDevice instance."))
+        devtype = dev.virtual_device_type
+
+        if   devtype == VirtualDevice.VIRTUAL_DEV_DISK:
+            self.disks.append(dev)
+        elif devtype == VirtualDevice.VIRTUAL_DEV_NET:
+            self.nics.append(dev)
+        elif devtype == VirtualDevice.VIRTUAL_DEV_AUDIO:
+            self.sound_devs.append(dev)
+        elif devtype == VirtualDevice.VIRTUAL_DEV_GRAPHICS:
+            self._graphics_dev = dev
+        elif devtype == VirtualDevice.VIRTUAL_DEV_HOSTDEV:
+            self.hostdevs.append(dev)
+        else:
+            self._devices.append(dev)
+
+    def get_devices(self, devtype):
+        """
+        Return a list of devices of type 'devtype' that will installed on
+        the guest.
+
+        @param devtype: Device type to search for (one of
+                        VirtualDevice.virtual_device_types)
+        """
+        if   devtype == VirtualDevice.VIRTUAL_DEV_DISK:
+            return self.disks[:]
+        elif devtype == VirtualDevice.VIRTUAL_DEV_NET:
+            return self.nics[:]
+        elif devtype == VirtualDevice.VIRTUAL_DEV_AUDIO:
+            return self.sound_devs[:]
+        elif devtype == VirtualDevice.VIRTUAL_DEV_GRAPHICS:
+            return self._graphics_dev and [self._graphics_dev] or []
+        elif devtype == VirtualDevice.VIRTUAL_DEV_HOSTDEV:
+            return self.hostdevs[:]
+        else:
+            return self._dev_build_list(devtype)
+
+    def get_all_devices(self):
+        """
+        Return a list of all devices being installed with the guest
+        """
+        retlist = []
+        for devtype in VirtualDevice.virtual_device_types:
+            retlist.extend(self.get_devices(devtype))
+        return retlist
+
+    def remove_device(self, dev):
+        """
+        Remove the passed device from the guest's device list
+
+        @param dev: VirtualDevice instance
+        """
+        if dev == self._graphics_dev:
+            self._graphics_dev = None
+
+        for devlist in [self.disks, self.nics, self.sound_devs, self.hostdevs,
+                        self._devices]:
+            if dev in devlist:
+                devlist.remove(dev)
+
+    # Device fetching functions used internally during the install process.
+    # These allow us to change dev defaults, add install media, etc. during
+    # the install, but revert to a clean state if the install fails
+    def _init_install_devs(self):
+        self._install_devices = self.get_all_devices()[:]
+
+    def _get_install_devs(self, devtype):
+        return self._dev_build_list(devtype, self._install_devices)
+
+    def _add_install_dev(self, dev):
+        self._install_devices.append(dev)
+
+    def _get_all_install_devs(self):
+        retlist = []
+        for devtype in VirtualDevice.virtual_device_types:
+            retlist.extend(self._get_install_devs(devtype))
+        return retlist
+
     # Private xml building methods
 
-    def _get_disk_xml(self, install=True):
-        """Return xml for disk devices (Must be implemented in subclass)"""
-        raise NotImplementedError
-
-    def _get_network_xml(self):
-        """Get the network config in the libvirt XML format"""
-        xml = ""
-        for n in self._install_nics:
-            xml = _util.xml_append(xml, n.get_xml_config())
-        return xml
-
-    def _get_graphics_xml(self):
-        """Get the graphics config in the libvirt XML format."""
-        if self._graphics_dev is None:
-            return ""
-        return self._graphics_dev.get_xml_config()
-
     def _get_input_device(self):
         """ Return a tuple of the form (devtype, bus) for the desired
             input device. (Must be implemented in subclass) """
         raise NotImplementedError
 
-    def _get_input_xml(self):
-        """Get the input device config in libvirt XML format."""
-        (devtype, bus) = self._get_input_device()
-        return "    <input type='%s' bus='%s'/>" % (devtype, bus)
-
-    def _get_sound_xml(self):
-        """Get the sound device configuration in libvirt XML format."""
-        xml = ""
-        for sound_dev in self.sound_devs:
-            xml = _util.xml_append(xml, sound_dev.get_xml_config())
-        return xml
-
-    def _get_hostdev_xml(self):
-        xml = ""
-        for hostdev in self.hostdevs:
-            xml = _util.xml_append(xml, hostdev.get_xml_config())
-        return xml
-
     def _get_device_xml(self, install=True):
+        change_disks = []
+        for d in self._get_install_devs(VirtualDevice.VIRTUAL_DEV_DISK):
+            if (d.device == VirtualDisk.DEVICE_CDROM
+                and d.transient
+                and not install):
+                # Keep cdrom around, but with no media attached,
+                # But only if we are a distro that doesn't have a multi
+                # stage install (aka not Windows)
+                if not self.get_continue_inst():
+                    change_disks.append((d, d.path))
+                    d.path = None
+
         xml = ""
+        try:
+            for dev in self._get_all_install_devs():
+                xml = _util.xml_append(xml, dev.get_xml_config())
+        finally:
+            try:
+                for disk, path in change_disks:
+                    disk.path = path
+            except:
+                pass
 
-        xml = _util.xml_append(xml, self._get_disk_xml(install))
-        xml = _util.xml_append(xml, self._get_network_xml())
-        xml = _util.xml_append(xml, self._get_input_xml())
-        xml = _util.xml_append(xml, self._get_graphics_xml())
-        xml = _util.xml_append(xml, self._get_sound_xml())
-        xml = _util.xml_append(xml, self._get_hostdev_xml())
         return xml
 
     def _get_features_xml(self):
@@ -500,31 +625,25 @@ class Guest(object):
         else:
             cpuset = ""
 
-        return """<domain type='%(type)s'>
-  <name>%(name)s</name>
-  <currentMemory>%(ramkb)s</currentMemory>
-  <memory>%(maxramkb)s</memory>
-  <uuid>%(uuid)s</uuid>
-  %(osblob)s
-  <on_poweroff>destroy</on_poweroff>
-  <on_reboot>%(action)s</on_reboot>
-  <on_crash>%(action)s</on_crash>
-  <vcpu%(cpuset)s>%(vcpus)d</vcpu>
-  <devices>
-%(devices)s
-  </devices>
-</domain>
-""" % { "type": self.type,
-        "name": self.name, \
-        "vcpus": self.vcpus, \
-        "cpuset": cpuset, \
-        "uuid": self.uuid, \
-        "ramkb": self.memory * 1024, \
-        "maxramkb": self.maxmemory * 1024, \
-        "devices": self._get_device_xml(install), \
-        "osblob": osblob, \
-        "action": action }
+        xml = ""
+        add = lambda x: _util.xml_append(xml, x)
 
+        xml = add("<domain type='%s'>" % self.type)
+        xml = add("  <name>%s</name>" % self.name)
+        xml = add("  <currentMemory>%s</currentMemory>" % (self.memory * 1024))
+        xml = add("  <memory>%s</memory>" % (self.maxmemory * 1024))
+        xml = add("  <uuid>%s</uuid>" % self.uuid)
+        xml = add("  %s" % osblob)
+        xml = add("  <on_poweroff>destroy</on_poweroff>")
+        xml = add("  <on_reboot>%s</on_reboot>" % action)
+        xml = add("  <on_crash>%s</on_crash>" % action)
+        xml = add("  <vcpu%s>%d</vcpu>" % (cpuset, self.vcpus))
+        xml = add("  <devices>")
+        xml = add("%s" % self._get_device_xml(install))
+        xml = add("  </devices>")
+        xml = add("</domain>\n")
+
+        return xml
 
     def start_install(self, consolecb=None, meter=None, removeOld=False,
                       wait=True):
@@ -586,22 +705,28 @@ class Guest(object):
 
 
     def _prepare_install(self, meter):
-        self._install_disks = self.disks[:]
-        self._install_nics = self.nics[:]
-        self._set_defaults()
+        self._init_install_devs()
 
         self._installer.prepare(guest = self,
                                 meter = meter)
         if self._installer.install_disk is not None:
-            self._install_disks.append(self._installer.install_disk)
+            self._add_install_dev(self._installer.install_disk)
+
+        # Run 'set_defaults' after install prep, since some installers
+        # (ImageInstaller) alter the device list.
+        self._set_defaults()
+
+        # Only set up an input device if there isn't already preset
+        if not self._get_install_devs(VirtualDevice.VIRTUAL_DEV_INPUT):
+            self._add_install_dev(self._get_input_device())
 
     def _create_devices(self, progresscb):
         """Ensure that devices are setup"""
-        for disk in self._install_disks:
+        for disk in self._get_install_devs(VirtualDevice.VIRTUAL_DEV_DISK):
             disk.setup(progresscb)
-        for nic in self._install_nics:
+        for nic in self._get_install_devs(VirtualDevice.VIRTUAL_DEV_NET):
             nic.setup(self.conn)
-        for hostdev in self.hostdevs:
+        for hostdev in self._get_install_devs(VirtualDevice.VIRTUAL_DEV_HOSTDEV):
             hostdev.setup()
 
     def _do_install(self, consolecb, meter, removeOld=False, wait=True):
@@ -696,6 +821,15 @@ class Guest(object):
             raise RuntimeError, _("Domain has already been started!")
 
     def _set_defaults(self):
+        used_targets = []
+        for disk in self._get_install_devs(VirtualDevice.VIRTUAL_DEV_DISK):
+            if not disk.bus:
+                if disk.device == disk.DEVICE_FLOPPY:
+                    disk.bus = "fdc"
+                else:
+                    disk.bus = self._diskbus
+            used_targets.append(disk.generate_target(used_targets))
+
         if self.uuid is None:
             while 1:
                 self.uuid = _util.uuidToString(_util.randomUUID())
@@ -716,38 +850,23 @@ class Guest(object):
         Using self.os_type and self.os_variant to find key in OSTYPES
         @returns: dict value, or None if os_type/variant wasn't set
         """
-        typ = self.os_type
-        var = self.os_variant
-        if typ:
-            if var and self._OS_TYPES[typ]["variants"][var].has_key(key):
-                return self._OS_TYPES[typ]["variants"][var][key]
-            elif self._OS_TYPES[typ].has_key(key):
-                return self._OS_TYPES[typ][key]
-        return self._DEFAULTS[key]
+        return osdict.lookup_osdict_key(self.conn, self.type, self.os_type,
+                                        self.os_variant, key)
 
     def _lookup_device_param(self, device_key, param):
         """
         Check the OS dictionary for the prefered device setting for passed
         device type and param (bus, model, etc.)
         """
-        os_devs = self._lookup_osdict_key("devices")
-        default_devs = self._DEFAULTS["devices"]
-        for devs in [os_devs, default_devs]:
-            if not devs.has_key(device_key):
-                continue
-            for ent in devs[device_key][param]:
-                hv_types = ent[0]
-                param_value = ent[1]
-                if self.type in hv_types:
-                    return param_value
-                elif "all" in hv_types:
-                    return param_value
-        raise RuntimeError(_("Invalid dictionary entry for device '%s %s'" % \
-                             (device_key, param)))
+        return osdict.lookup_device_param(self.conn, self.type, self.os_type,
+                                          self.os_variant, device_key, param)
 
     def terminate_console(self):
         if self._consolechild:
-            os.kill(self._consolechild, signal.SIGKILL)
+            try:
+                os.kill(self._consolechild, signal.SIGKILL)
+            except:
+                pass
 
 def _wait_for_domain(conn, name):
     # sleep in .25 second increments until either a) we get running
