@@ -38,10 +38,22 @@ class ImageFetcher:
         self.location = location
         self.scratchdir = scratchdir
 
+    def _make_path(self, filename):
+        if hasattr(self, "srcdir"):
+            path = getattr(self, "srcdir")
+        else:
+            path = self.location
+
+        if not path.endswith("/"):
+            path += "/"
+        path += filename
+        return path
+
     def saveTemp(self, fileobj, prefix):
         if not os.path.exists(self.scratchdir):
             os.makedirs(self.scratchdir, 0750)
-        (fd, fn) = tempfile.mkstemp(prefix="virtinst-" + prefix, dir=self.scratchdir)
+        (fd, fn) = tempfile.mkstemp(prefix="virtinst-" + prefix,
+                                    dir=self.scratchdir)
         block_size = 16384
         try:
             while 1:
@@ -59,8 +71,30 @@ class ImageFetcher:
     def cleanupLocation(self):
         pass
 
-    def acquireFile(self, src, progresscb):
-        raise NotImplementedError("Must be implemented in subclass")
+    def acquireFile(self, filename, progresscb):
+        # URLGrabber works for all network and local cases
+
+        f = None
+        try:
+            path = self._make_path(filename)
+            base = os.path.basename(filename)
+            logging.debug("Fetching URI: %s" % path)
+
+            try:
+                f = grabber.urlopen(path,
+                                    progress_obj = progresscb,
+                                    text = _("Retrieving file %s...") % base)
+            except Exception, e:
+                raise ValueError, _("Couldn't acquire file %s: %s") % \
+                                    (path, str(e))
+
+            tmpname = self.saveTemp(f, prefix=base + ".")
+            logging.debug("Saved file to " + tmpname)
+            return tmpname
+        finally:
+            if f:
+                f.close()
+
 
     def hasFile(self, src):
         raise NotImplementedError("Must be implemented in subclass")
@@ -68,13 +102,6 @@ class ImageFetcher:
 # Base class for downloading from FTP / HTTP
 class URIImageFetcher(ImageFetcher):
 
-    def _make_path(self, filename):
-        path = self.location
-        if not path.endswith("/"):
-            path += "/"
-        path += filename
-        return path
-
     def hasFile(self, filename):
         raise NotImplementedError
 
@@ -83,26 +110,6 @@ class URIImageFetcher(ImageFetcher):
             raise ValueError(_("Opening URL %s failed.") % \
                               (self.location))
 
-    def acquireFile(self, filename, progresscb):
-        f = None
-        try:
-            path = self._make_path(filename)
-            base = os.path.basename(filename)
-            logging.debug("Fetching URI " + path)
-            try:
-                f = grabber.urlopen(path,
-                                    progress_obj = progresscb,
-                                    text = _("Retrieving file %s...") % base)
-            except IOError, e:
-                raise ValueError, _("Couldn't acquire file %s: %s") % \
-                                    (path, str(e))
-            tmpname = self.saveTemp(f, prefix=base + ".")
-            logging.debug("Saved file to " + tmpname)
-            return tmpname
-        finally:
-            if f:
-                f.close()
-
 class HTTPImageFetcher(URIImageFetcher):
 
     def hasFile(self, filename):
@@ -141,36 +148,12 @@ class LocalImageFetcher(ImageFetcher):
         ImageFetcher.__init__(self, location, scratchdir)
         self.srcdir = srcdir
 
-    def acquireFile(self, filename, progresscb):
-        f = None
-        try:
-            logging.debug("Acquiring file from " + self.srcdir + "/" + filename)
-            base = os.path.basename(filename)
-            try:
-                src = self.srcdir + "/" + filename
-                if stat.S_ISDIR(os.stat(src)[stat.ST_MODE]):
-                    logging.debug("Found a directory")
-                    return None
-                else:
-                    f = open(src, "r")
-            except IOError, e:
-                raise ValueError, _("Invalid file location given: ") + str(e)
-            except OSError, (errno, msg):
-                raise ValueError, \
-                      _("Invalid file location given: %s: %s") % (errno, msg)
-            tmpname = self.saveTemp(f, prefix=base + ".")
-            logging.debug("Saved file to " + tmpname)
-            return tmpname
-        finally:
-            if f:
-                f.close()
-
     def hasFile(self, filename):
-        if os.path.exists(os.path.abspath(self.srcdir + "/" + filename)):
+        src = self._make_path(filename)
+        if os.path.exists(src):
             return True
         else:
-            logging.debug("local hasFile: Couldn't find %s" % \
-                          (self.srcdir + "/" + filename))
+            logging.debug("local hasFile: Couldn't find %s" % src)
             return False
 
 # This is a fetcher capable of extracting files from a NFS server
@@ -179,7 +162,8 @@ class MountedImageFetcher(LocalImageFetcher):
 
     def prepareLocation(self):
         cmd = None
-        self.srcdir = tempfile.mkdtemp(prefix="virtinstmnt.", dir=self.scratchdir)
+        self.srcdir = tempfile.mkdtemp(prefix="virtinstmnt.",
+                                       dir=self.scratchdir)
         mountcmd = "/bin/mount"
         if os.uname()[0] == "SunOS":
             mountcmd = "/usr/sbin/mount"
@@ -200,8 +184,8 @@ class MountedImageFetcher(LocalImageFetcher):
         ret = subprocess.call(cmd)
         if ret != 0:
             self.cleanupLocation()
-            logging.debug("Mounting location %s failed" % (self.location,))
-            raise ValueError(_("Mounting location %s failed") % (self.location))
+            raise ValueError(_("Mounting location '%s' failed") %
+                             (self.location))
         return True
 
     def cleanupLocation(self):
