@@ -28,6 +28,7 @@ import socket
 import ConfigParser
 
 import virtinst
+from virtinst.Guest import Guest
 from virtinst import _util
 from virtinst import _virtinst as _
 
@@ -36,6 +37,13 @@ from ImageFetcher import FTPImageFetcher
 from ImageFetcher import HTTPImageFetcher
 from ImageFetcher import DirectImageFetcher
 
+def safeint(c):
+    try:
+        val = int(c)
+    except:
+        val = 0
+    return val
+
 def _fetcherForURI(uri, scratchdir=None):
     if uri.startswith("http://"):
         fclass = HTTPImageFetcher
@@ -86,6 +94,8 @@ def _storeForDistro(fetcher, baseuri, typ, progresscb, arch, distro=None,
         stores.append(SolarisDistro)
     if distro == "solaris" or distro is None:
         stores.append(OpenSolarisDistro)
+    if distro == "netware" or distro is None:
+        stores.append(NetWareDistro)
 
     stores.append(GenericDistro)
 
@@ -116,8 +126,9 @@ def _acquireMedia(iskernel, guest, baseuri, progresscb, arch,
         if iskernel is True:
             # FIXME: We should probably do this for both kernel and boot
             # disk?
+            os_type, os_variant = store.get_osdict_info()
             return (store.acquireKernel(guest, fetcher, progresscb),
-                    store.os_type, store.os_variant)
+                    os_type, os_variant)
         elif iskernel is False:
             return store.acquireBootDisk(fetcher, progresscb)
         else:
@@ -138,12 +149,18 @@ def acquireBootDisk(baseuri, progresscb, arch, scratchdir="/var/tmp",
     return _acquireMedia(False, None, baseuri, progresscb, arch,
                          scratchdir, type, distro)
 
+def _check_ostype_valid(os_type):
+    return bool(os_type in Guest.list_os_types())
+def _check_osvariant_valid(os_type, os_variant):
+    return bool(_check_ostype_valid(os_type) and
+                os_variant in Guest.list_os_variants(os_type))
+
 # Attempt to detect the os type + variant for the passed location
 def detectMediaDistro(location, arch):
     import urlgrabber
     progress = urlgrabber.progress.BaseMeter()
     store = _acquireMedia(None, None, location, progress, arch, "/var/tmp")
-    return (store.os_type, store.os_variant)
+    return store.get_osdict_info()
 
 
 def distroFromTreeinfo(fetcher, progresscb, uri, arch, vmtype=None,
@@ -160,7 +177,10 @@ def distroFromTreeinfo(fetcher, progresscb, uri, arch, vmtype=None,
     finally:
         os.unlink(tmptreeinfo)
 
-    fam = treeinfo.get("general", "family")
+    try:
+        fam = treeinfo.get("general", "family")
+    except ConfigParser.NoSectionError:
+        return None
 
     if re.match(".*Fedora.*", fam):
         dclass = FedoraDistro
@@ -236,7 +256,8 @@ class Distro:
 
     def acquireBootDisk(self, fetcher, progresscb):
         if self._hasTreeinfo(fetcher, progresscb):
-            return fetcher.acquireFile(self._getTreeinfoMedia("boot.iso"))
+            return fetcher.acquireFile(self._getTreeinfoMedia("boot.iso"),
+                                       progresscb)
         else:
             for path in self._boot_iso_paths:
                 if fetcher.hasFile(path):
@@ -244,6 +265,30 @@ class Distro:
             raise RuntimeError(_("Could not find boot.iso in %s tree." % \
                                self.name))
 
+    def get_osdict_info(self):
+        """
+        Return (distro, variant) tuple, checking to make sure they are valid
+        osdict entries
+        """
+        if not self.os_type:
+            return (None, None)
+
+        if not _check_ostype_valid(self.os_type):
+            logging.debug("%s set os_type to %s, which is not in osdict." %
+                          (self, self.os_type))
+            return (None, None)
+
+        if not self.os_variant:
+            return (self.os_type, None)
+
+        if not _check_osvariant_valid(self.os_type, self.os_variant):
+            logging.debug("%s set os_variant to %s, which is not in osdict"
+                          " for distro %s." %
+                          (self, self.os_variant, self.os_type))
+            return (self.os_type, None)
+
+        return (self.os_type, self.os_variant)
+
     def _hasTreeinfo(self, fetcher, progresscb):
         # all Red Hat based distros should have .treeinfo, perhaps others
         # will in time
@@ -427,7 +472,7 @@ class FedoraDistro(RedHatDistro):
                 if ver == "development":
                     self.os_variant = self._latestFedoraVariant()
                 elif ver:
-                    self.os_variant = "fedora" + str(ver)
+                    self.os_variant = "fedora" + (str(ver).split("-"))[0]
 
             return ret
         else:
@@ -475,15 +520,49 @@ class RHELDistro(RedHatDistro):
                 return True
             return False
 
+    def _parseTreeinfoVersion(self, verstr):
+        version = safeint(verstr[0])
+        update = 0
+
+        updinfo = verstr.split(".")
+        if len(updinfo) > 1:
+            update = safeint(updinfo[1])
+
+        return version, update
+
     def _variantFromVersion(self):
-        # XXX: Version will look like '5.2', rather than just '5'. We
-        # need to support this in the os dictionary at some point, but for
-        # now, just use the '5' part
         ver = self.treeinfo.get("general", "version")
         if not ver:
             return
 
-        self.os_variant = "rhel" + ver[0]
+        version, update = self._parseTreeinfoVersion(ver)
+        self._setRHELVariant(version, update)
+
+    def _setRHELVariant(self, version, update):
+        if not _check_ostype_valid(self.os_type):
+            return
+
+        base = "rhel" + str(version)
+        if update < 0:
+            update = 0
+
+        ret = None
+        while update >= 0:
+            tryvar = base + ".%s" % update
+            if not _check_osvariant_valid(self.os_type, tryvar):
+                update -= 1
+                continue
+
+            ret = tryvar
+            break
+
+        if not ret:
+            # Try plain rhel5, rhel6, whatev
+            if _check_osvariant_valid(self.os_type, base):
+                ret = base
+
+        if ret:
+            self.os_variant = ret
 
 
 # CentOS distro check
@@ -531,6 +610,16 @@ class SLDistro(RHELDistro):
                 return True
             return False
 
+    def _parseTreeinfoVersion(self, verstr):
+        """
+        Overrides method in RHELDistro
+        """
+        version = safeint(verstr[0])
+        update = 0
+
+        if len(verstr) > 1:
+            update = safeint(verstr[1])
+        return version, update
 
 
 # Suse  image store is harder - we fetch the kernel RPM and a helper
@@ -540,8 +629,6 @@ class SuseDistro(Distro):
     name = "SUSE"
     os_type = "linux"
     _boot_iso_paths   = [ "boot/boot.iso" ]
-    _hvm_kernel_paths = []
-    _xen_kernel_paths = []
 
     def __init__(self, arch, uri, vmtype=None, scratchdir=None):
         Distro.__init__(self, arch, uri, vmtype, scratchdir)
@@ -549,11 +636,11 @@ class SuseDistro(Distro):
             self.arch = 'i386'
 
         # Tested with Opensuse 10, 11, and sles 10
-        self._hvm_kernel_paths += [ ("boot/%s/loader/linux" % self.arch,
-                                     "boot/%s/loader/initrd" % self.arch) ]
+        self._hvm_kernel_paths = [ ("boot/%s/loader/linux" % self.arch,
+                                    "boot/%s/loader/initrd" % self.arch) ]
         # Matches Opensuse > 10.2 and sles 10
-        self._xen_kernel_paths += [ ("boot/%s/vmlinuz-xen" % self.arch,
-                                     "boot/%s/initrd-xen" % self.arch) ]
+        self._xen_kernel_paths = [ ("boot/%s/vmlinuz-xen" % self.arch,
+                                    "boot/%s/initrd-xen" % self.arch) ]
 
     def isValidStore(self, fetcher, progresscb):
         # Suse distros always have a 'directory.yast' file in the top
@@ -1022,8 +1109,11 @@ class SolarisDistro(SunDistro):
                 self.initrdpath)
 
 class OpenSolarisDistro(SunDistro):
+
+    os_variant = "opensolaris"
+ 
     kernelpath = "platform/i86xpv/kernel/unix"
-    initrdpath = "boot/x86.microroot"
+    initrdpaths = [ "platform/i86pc/boot_archive", "boot/x86.microroot" ]
 
     def isValidStore(self, fetcher, progresscb):
         if fetcher.hasFile(self.kernelpath):
@@ -1042,7 +1132,7 @@ class OpenSolarisDistro(SunDistro):
 
         args = ''
         if kopts:
-            args += '-' + kopts
+            args += ' -' + kopts
         if kbargs:
             args += ' -B ' + kbargs
 
@@ -1059,9 +1149,31 @@ class OpenSolarisDistro(SunDistro):
         args = "/" + self.kernelpath + self.install_args(guest)
 
         try:
-            initrd = fetcher.acquireFile(self.initrdpath, progresscb)
+            initrd = fetcher.acquireFile(self.initrdpaths[0], progresscb)
             return (kernel, initrd, args)
-        except:
-            os.unlink(kernel)
-            raise RuntimeError(_("OpenSolaris microroot not found at %s") %
-                self.initrdpath)
+        except Exception, e:
+            try:
+                initrd = fetcher.acquireFile(self.initrdpaths[1], progresscb)
+                return (kernel, initrd, args)
+            except:
+                os.unlink(kernel)
+                raise Exception("No OpenSolaris boot archive found: %s\n" % e)
+
+
+# NetWare 6 PV
+class NetWareDistro(Distro):
+    name = "NetWare"
+    os_type = "other"
+    os_variant = "netware6"
+
+    loaderpath = "STARTUP/XNLOADER.SYS"
+
+    def isValidStore(self, fetcher, progresscb):
+        if fetcher.hasFile(self.loaderpath):
+            logging.debug("Detected NetWare")
+            return True
+        return False
+
+    def acquireKernel(self, guest, fetcher, progresscb):
+        loader = fetcher.acquireFile(self.loaderpath, progresscb)
+        return (loader, "", "")
