@@ -21,6 +21,10 @@
 
 from Guest import Guest
 from DistroInstaller import DistroInstaller
+from VirtualInputDevice import VirtualInputDevice
+from VirtualDevice import VirtualDevice
+from VirtualDisk import VirtualDisk
+import _util
 
 class ParaVirtGuest(Guest):
     def __init__(self, type=None, connection=None, hypervisorURI=None,
@@ -30,24 +34,20 @@ class ParaVirtGuest(Guest):
                                         conn=connection)
         Guest.__init__(self, type, connection, hypervisorURI, installer)
         self.disknode = "xvd"
+        self._diskbus = "xen"
 
     def _get_input_device(self):
-        return ("mouse", "xen")
+        dev = VirtualInputDevice(self.conn)
+        dev.type = "mouse"
+        dev.bus = "xen"
+        return dev
 
-    def _get_disk_xml(self, install = True):
-        """Get the disk config in the libvirt XML format"""
-        ret = ""
-        used_targets = []
-        for disk in self._install_disks:
-            if not disk.bus:
-                disk.bus = "xen"
-            used_targets.append(disk.generate_target(used_targets))
+    def _set_defaults(self):
+        # Default file backed PV guests to tap driver
+        for d in self._get_install_devs(VirtualDevice.VIRTUAL_DEV_DISK):
+            if (d.type == VirtualDisk.TYPE_FILE
+                and _util.is_blktap_capable()
+                and d.driver_name == None):
+                d.driver_name = VirtualDisk.DRIVER_TAP
 
-        for d in self._install_disks:
-            if d.transient and not install:
-                continue
-
-            if ret:
-                ret += "\n"
-            ret += d.get_xml_config(d.target)
-        return ret
+        Guest._set_defaults(self)
