@@ -45,27 +45,65 @@ General workflow for the different storage objects:
 @see: U{http://libvirt.org/storage.html}
 """
 
-
 import libvirt
+import threading
+import time
 
 import logging
 from xml.sax.saxutils import escape
 
 import _util
+import support
 from virtinst import _virtinst as _
 
 DEFAULT_DEV_TARGET = "/dev"
 DEFAULT_LVM_TARGET_BASE = "/dev/"
 DEFAULT_DIR_TARGET_BASE = "/var/lib/libvirt/images/"
-DEFAULT_ISCSI_TARGET = "/dev/disk/by-path"
+DEFAULT_SCSI_TARGET = "/dev/disk/by-path"
 
-# Pools:
-#   DirectoryPool         : A flat filesystem directory
-#   FilesystemPool        : A formatted partition
-#   NetworkFilesystemPool : NFS
-#   LogicalPool           : LVM Volume Group
-#   DiskPool              : Raw disk
-#   iSCSIPool             : iSCSI
+# Pulled from libvirt, used for building on older versions
+VIR_STORAGE_VOL_FILE = 0
+VIR_STORAGE_VOL_BLOCK = 1
+
+def is_create_vol_from_supported(conn):
+    return support.check_pool_support(conn,
+                                      support.SUPPORT_STORAGE_CREATEVOLFROM)
+
+def _parse_pool_source_list(source_xml):
+    def source_parser(node):
+        ret_list = []
+
+        child = node.children
+        while child:
+            if child.name == "source":
+                val_dict = {}
+                source = child.children
+
+                while source:
+                    if source.name == "name":
+                        val_dict["source_name"] = source.content
+                    elif source.name == "host":
+                        val_dict["host"] = source.prop("name")
+                    elif source.name == "format":
+                        val_dict["format"] = source.prop("type")
+                    elif source.name in ["device", "dir"]:
+                        val_dict["source_path"] = source.prop("path")
+                    source = source.next
+
+                ret_list.append(val_dict)
+
+            child = child.next
+
+        for val_dict in ret_list:
+            if (val_dict.get("format") == "lvm2" and
+                val_dict.get("source_name") and
+                not val_dict.get("target_path")):
+                val_dict["target_path"] = (DEFAULT_LVM_TARGET_BASE +
+                                           val_dict["source_name"])
+
+        return ret_list
+
+    return _util.parse_node_helper(source_xml, "sources", source_parser)
 
 class StorageObject(object):
     """
@@ -130,9 +168,9 @@ class StorageObject(object):
     def set_perms(self, val):
         if type(val) is not dict:
             raise ValueError(_("Permissions must be passed as a dict object"))
-        for key in ["mode", "owner", "group", "label"]:
+        for key in ["mode", "owner", "group"]:
             if not key in val:
-                raise ValueError(_("Permissions must contain 'mode', 'owner', 'group' and 'label' keys."))
+                raise ValueError(_("Permissions must contain 'mode', 'owner' and 'group' keys."))
         self._perms = val
 
 
@@ -142,6 +180,7 @@ class StorageObject(object):
             raise ValueError(_("'%s' is not an absolute path." % path))
 
     def _check_name_collision(self, name):
+        ignore = name
         raise RuntimeError, "Must be implemented in subclass"
 
     # XML Building
@@ -155,12 +194,16 @@ class StorageObject(object):
         perms = self.get_perms()
         if not perms:
             return ""
-        return "    <permissions>\n" + \
-               "      <mode>%o</mode>\n" % perms["mode"] + \
+        xml =  "    <permissions>\n" + \
+               "      <mode>0%o</mode>\n" % perms["mode"] + \
                "      <owner>%d</owner>\n" % perms["owner"] + \
-               "      <group>%d</group>\n" % perms["group"] + \
-               "      <label>%s</label>\n" % perms["label"] + \
-               "    </permissions>\n"
+               "      <group>%d</group>\n" % perms["group"]
+
+        if perms.has_key("label"):
+            xml += "      <label>%s</label>\n" % perms["label"]
+
+        xml += "    </permissions>\n"
+        return xml
 
 
     def get_xml_config(self):
@@ -179,7 +222,7 @@ class StorageObject(object):
         xml = "%s" % (root_xml) + \
               """  <name>%s</name>\n""" % (self.name) + \
               """%(stor_xml)s""" % { "stor_xml" : self._get_storage_xml() } + \
-              """</%s>""" % (self.object_type)
+              """</%s>\n""" % (self.object_type)
         return xml
 
 
@@ -196,6 +239,7 @@ class StoragePool(StorageObject):
     TYPE_LOGICAL = "logical"
     TYPE_DISK    = "disk"
     TYPE_ISCSI   = "iscsi"
+    TYPE_SCSI   = "scsi"
     """@group Types: TYPE_*"""
 
     # Pool type descriptions for use in higher level programs
@@ -206,6 +250,7 @@ class StoragePool(StorageObject):
     _types[TYPE_LOGICAL] = _("LVM Volume Group")
     _types[TYPE_DISK]    = _("Physical Disk Device")
     _types[TYPE_ISCSI]   = _("iSCSI Target")
+    _types[TYPE_SCSI]    = _("SCSI host adapter")
 
     def get_pool_class(ptype):
         """
@@ -228,6 +273,8 @@ class StoragePool(StorageObject):
             return DiskPool
         if ptype == StoragePool.TYPE_ISCSI:
             return iSCSIPool
+        if ptype == StoragePool.TYPE_SCSI:
+            return SCSIPool
     get_pool_class = staticmethod(get_pool_class)
 
     def get_volume_for_pool(pool_type):
@@ -246,6 +293,50 @@ class StoragePool(StorageObject):
         return StoragePool._types[pool_type]
     get_pool_type_desc = staticmethod(get_pool_type_desc)
 
+    def pool_list_from_sources(conn, name, pool_type, host=None):
+        """
+        Return a list of StoragePool instances built from libvirt's pool
+        source enumeration (if supported).
+
+        @param conn: Libvirt connection
+        @param name: Name for the new pool
+        @param pool_type: Pool type string from L{Types}
+        @param host: Option host string to poll for sources
+        """
+        if not support.check_conn_support(conn,
+                                     support.SUPPORT_CONN_FINDPOOLSOURCES):
+            return []
+
+        pool_class = StoragePool.get_pool_class(pool_type)
+        pool_inst = pool_class(conn=conn, name=name)
+
+        if host:
+            source_xml = "<source><host name='%s'/></source>" % host
+        else:
+            source_xml = "<source/>"
+
+        try:
+            xml = conn.findStoragePoolSources(pool_type, source_xml, 0)
+        except libvirt.libvirtError, e:
+            if support.is_error_nosupport(e):
+                return []
+            raise
+
+        retlist = []
+        source_list = _parse_pool_source_list(xml)
+        for source in source_list:
+            pool_inst = pool_class(conn=conn, name=name)
+            for key, val in source.items():
+
+                if not hasattr(pool_inst, key):
+                    continue
+
+                setattr(pool_inst, key, val)
+
+            retlist.append(pool_inst)
+
+        return retlist
+    pool_list_from_sources = staticmethod(pool_list_from_sources)
 
     def __init__(self, conn, name, type, target_path=None, uuid=None):
         StorageObject.__init__(self, object_type=StorageObject.TYPE_POOL, \
@@ -258,16 +349,18 @@ class StoragePool(StorageObject):
         self._host = None
         self._format = None
         self._source_path = None
+        self._uuid = None
 
         if target_path is None:
             target_path = self._get_default_target_path()
         self.target_path = target_path
 
+        if uuid:
+            self.uuid = uuid
+
         # Initialize all optional properties
         self._host = None
         self._source_path = None
-        if not uuid:
-            self._uuid = None
         self._random_uuid = _util.uuidToString(_util.randomUUID())
 
     # Properties used by all pools
@@ -483,7 +576,7 @@ class NetworkFilesystemPool(StoragePool):
         return FileVolume
     get_volume_class = staticmethod(get_volume_class)
 
-    formats = [ "auto", "nfs" ]
+    formats = [ "auto", "nfs", "glusterfs"]
 
     # Register applicable property methods from parent class
     source_path = property(StoragePool.get_source_path,
@@ -548,11 +641,65 @@ class LogicalPool(StoragePool):
                            StoragePool.set_target_path,
                            doc=_("Location of the existing LVM volume group."))
 
-    def __init__(self, conn, name, target_path=None, uuid=None, perms=None):
+    def __init__(self, conn, name, target_path=None, uuid=None, perms=None,
+                 source_path=None, source_name=None):
         StoragePool.__init__(self, name=name, type=StoragePool.TYPE_LOGICAL,
                              target_path=target_path, uuid=uuid, conn=conn)
+
+        self._source_name = None
+
         if perms:
             self.perms = perms
+        if source_path:
+            self.source_path = source_path
+        if source_name:
+            self.source_name = source_name
+
+    # Need to overwrite storage path checks, since this optionally be a list
+    # of devices
+    def get_source_path(self):
+        return self._source_path
+    def set_source_path(self, val):
+        if not val:
+            self._source_path = None
+            return
+
+        if type(val) != list:
+            StoragePool.set_source_path(self, val)
+        else:
+            self._source_path = val
+    source_path = property(get_source_path, set_source_path,
+                           doc=_("Optional device(s) to build new LVM volume "
+                                 "on."))
+
+    def get_source_name(self):
+        if self._source_name:
+            return self._source_name
+
+        # If a source name isn't explictly set, try to determine it from
+        # existing parameters
+        srcname = self.name
+
+        if (self.target_path and
+            self.target_path.startswith(DEFAULT_LVM_TARGET_BASE)):
+            # If there is a target path, parse it for an expected VG
+            # location, and pull the name from there
+            vg = self.target_path[len(DEFAULT_LVM_TARGET_BASE):]
+            srcname = vg.split("/", 1)[0]
+
+        return srcname
+
+    def set_source_name(self, val):
+        self._source_name = val
+    source_name = property(get_source_name, set_source_name,
+                           doc=_("Name of the Volume Group"))
+
+    def _make_source_name(self):
+        srcname = self.name
+
+        if self.source_path:
+            # Building a pool, so just use pool name
+            return srcname
 
     def _get_default_target_path(self):
         return DEFAULT_LVM_TARGET_BASE + self.name
@@ -563,12 +710,31 @@ class LogicalPool(StoragePool):
         return xml
 
     def _get_source_xml(self):
-        return ""
+        sources = self.source_path
+        if type(sources) != list:
+            sources = sources and [sources] or []
+
+        xml = ""
+        for s in sources:
+            xml += "    <device path='%s'/>\n" % s
+        if self.source_name:
+            xml += "    <name>%s</name>\n" % self.source_name
+        return xml
+
+    def install(self, meter=None, create=False, build=False):
+        if build and not self.source_path:
+            raise ValueError(_("Must explicitly specify source path if "
+                               "building' pool"))
+        return StoragePool.install(self, meter=meter, create=create,
+                                   build=build)
 
 class DiskPool(StoragePool):
     """
     Create a storage pool from a physical disk
     """
+    def get_volume_class():
+        return DiskVolume
+    get_volume_class = staticmethod(get_volume_class)
 
     # Register applicable property methods from parent class
     source_path = property(StoragePool.get_source_path,
@@ -581,10 +747,6 @@ class DiskPool(StoragePool):
 
     formats = [ "auto", "bsd", "dos", "dvh", "gpt", "mac", "pc98", "sun" ]
 
-    def get_volume_class():
-        raise NotImplementedError(_("Disk volume creation is not implemented."))
-    get_volume_class = staticmethod(get_volume_class)
-
     def __init__(self, conn, name, source_path=None, target_path=None,
                  format="auto", uuid=None):
         StoragePool.__init__(self, name=name, type=StoragePool.TYPE_DISK,
@@ -665,7 +827,7 @@ class iSCSIPool(StoragePool):
                            doc=_("Path on the host that is being shared."))
 
     def _get_default_target_path(self):
-        return DEFAULT_ISCSI_TARGET
+        return DEFAULT_SCSI_TARGET
 
     def _get_target_xml(self):
         xml = "    <path>%s</path>\n" % escape(self.target_path)
@@ -680,6 +842,57 @@ class iSCSIPool(StoragePool):
               """    <device path="%s"/>\n""" % escape(self.source_path)
         return xml
 
+class SCSIPool(StoragePool):
+    """
+    Create a SCSI based storage pool
+    """
+
+    target_path = property(StoragePool.get_target_path,
+                           StoragePool.set_target_path,
+                           doc=_("Root location for identifying new storage"
+                                 " volumes."))
+
+    def get_volume_class():
+        raise NotImplementedError(_("SCSI volume creation is not "
+                                    "implemented."))
+    get_volume_class = staticmethod(get_volume_class)
+
+    def __init__(self, conn, name, source_path=None,
+                 target_path=None, uuid=None):
+        StoragePool.__init__(self, name=name, type=StoragePool.TYPE_SCSI,
+                             uuid=uuid, target_path=target_path, conn=conn)
+
+        if source_path:
+            self.source_path = source_path
+
+    # Need to overwrite pool *_source_path since iscsi device isn't
+    # a fully qualified path
+    def get_source_path(self):
+        return self._source_path
+    def set_source_path(self, val):
+        self._source_path = val
+    source_path = property(get_source_path, set_source_path,
+                           doc=_("Name of the scsi adapter (ex. host2)"))
+
+    def _get_default_target_path(self):
+        return DEFAULT_SCSI_TARGET
+
+    def _get_target_xml(self):
+        xml = "    <path>%s</path>\n" % escape(self.target_path)
+        return xml
+
+    def _get_source_xml(self):
+        if not self.source_path:
+            raise RuntimeError(_("Adapter name is required"))
+        xml = """    <adapter name="%s"/>\n""" % escape(self.source_path)
+        return xml
+
+
+
+"""
+Storage Volume classes
+"""
+
 class StorageVolume(StorageObject):
     """
     Base class for building and installing libvirt storage volume xml
@@ -687,8 +900,19 @@ class StorageVolume(StorageObject):
 
     formats = []
 
+    # File vs. Block for the Volume class
+    _file_type = None
+
     def __init__(self, name, capacity, conn=None, pool_name=None, pool=None,
                  allocation=0):
+        """
+        @param name: Name for the new storage volume
+        @param capacity: Total size of the new volume (in bytes)
+        @param conn: optional virConnect instance to lookup pool_name on
+        @param pool_name: optional pool_name to install on
+        @param pool: virStoragePool object to install on
+        @param allocation: amount of storage to actually allocate (default 0)
+        """
         if pool is None:
             if pool_name is None:
                 raise ValueError(_("One of pool or pool_name must be "
@@ -706,10 +930,15 @@ class StorageVolume(StorageObject):
         self._allocation = None
         self._capacity = None
         self._format = None
+        self._input_vol = None
 
         self.allocation = allocation
         self.capacity = capacity
 
+        # Indicate that the volume installation has finished. Used to
+        # definitively tell the storage progress thread to stop polling.
+        self._install_finished = True
+
     def get_volume_for_pool(pool_object=None, pool_name=None, conn=None):
         """
         Returns volume class associated with passed pool_object/name
@@ -743,17 +972,8 @@ class StorageVolume(StorageObject):
                                                         pool_name=pool_name,
                                                         conn=conn)
         pool_object.refresh(0)
-
-        for i in range(0, 100000):
-            tryname = name
-            if i != 0:
-                tryname += ("-%d" % i)
-            tryname += suffix
-            try:
-                pool_object.storageVolLookupByName(tryname)
-            except libvirt.libvirtError:
-                return tryname
-        raise ValueError(_("Default volume target path range exceeded."))
+        return _util.generate_name(name, pool_object.storageVolLookupByName,
+                                   suffix)
     find_free_name = staticmethod(find_free_name)
 
     def lookup_pool_by_name(pool_object=None, pool_name=None, conn=None):
@@ -783,8 +1003,11 @@ class StorageVolume(StorageObject):
         return pool_object
     lookup_pool_by_name = staticmethod(lookup_pool_by_name)
 
-
     # Properties used by all volumes
+    def get_file_type(self):
+        return self._file_type
+    file_type = property(get_file_type)
+
     def get_capacity(self):
         return self._capacity
     def set_capacity(self, val):
@@ -836,6 +1059,23 @@ class StorageVolume(StorageObject):
         self._pool = newpool
     pool = property(get_pool, set_pool)
 
+    def get_input_vol(self):
+        return self._input_vol
+    def set_input_vol(self, vol):
+        if vol is None:
+            self._input_vol = None
+            return
+
+        if not isinstance(vol, libvirt.virStorageVol):
+            raise ValueError(_("input_vol must be a virStorageVol"))
+        if not is_create_vol_from_supported(self.pool._conn):
+            raise ValueError(_("Creating storage from an existing volume is"
+                               " not supported by this libvirt version."))
+        self._input_vol = vol
+    input_vol = property(get_input_vol, set_input_vol,
+                         doc=_("virStorageVolume pointer to clone/use as "
+                               "input."))
+
     # Property functions used by more than one child class
     def get_format(self):
         return self._format
@@ -892,22 +1132,69 @@ class StorageVolume(StorageObject):
         xml = self.get_xml_config()
         logging.debug("Creating storage volume '%s' with xml:\n%s" % \
                       (self.name, xml))
-        if meter:
-            #meter.start(size=self.capacity,
-            #            text=_("Creating storage volume..."))
-            # XXX: We don't have any meaningful way to update the meter
-            # XXX: throughout the operation, so just skip it
-            pass
+
+        t = threading.Thread(target=self._progress_thread,
+                             name="Checking storage allocation",
+                             args=(meter,))
+        t.setDaemon(True)
+
         try:
-            vol = self.pool.createXML(xml, 0)
-        except Exception, e:
-            raise RuntimeError("Couldn't create storage volume '%s': '%s'" %
-                               (self.name, str(e)))
-        if meter:
-            #meter.end(0)
-            pass
-        logging.debug("Storage volume '%s' install complete." % self.name)
-        return vol
+            try:
+                self._install_finished = False
+                t.start()
+                if meter:
+                    meter.start(size=self.allocation,
+                                text=_("Allocating '%s'") % self.name)
+
+                if self.input_vol:
+                    vol = self.pool.createXMLFrom(xml, self.input_vol, 0)
+                else:
+                    vol = self.pool.createXML(xml, 0)
+
+                if meter:
+                    meter.end(self.allocation)
+                logging.debug("Storage volume '%s' install complete." %
+                              self.name)
+                return vol
+            except libvirt.libvirtError, e:
+                if support.is_error_nosupport(e):
+                    raise RuntimeError("Libvirt version does not support "
+                                       "storage cloning.")
+            except Exception, e:
+                raise RuntimeError("Couldn't create storage volume "
+                                   "'%s': '%s'" % (self.name, str(e)))
+        finally:
+            self._install_finished = True
+
+    def _progress_thread(self, meter):
+        lookup_attempts = 10
+        vol = None
+
+        if not meter:
+            return
+
+        while lookup_attempts > 0:
+            try:
+                vol = self.pool.storageVolLookupByName(self.name)
+                break
+            except:
+                lookup_attempts -= 1
+                time.sleep(.2)
+                if self._install_finished:
+                    break
+                else:
+                    continue
+            break
+
+        if vol == None:
+            logging.debug("Couldn't lookup storage volume in prog thread.")
+            return
+
+        while not self._install_finished:
+            time.sleep(1)
+            ignore, ignore, alloc = vol.info()
+            meter.update(alloc)
+
 
     def is_size_conflict(self):
         """
@@ -939,6 +1226,7 @@ class FileVolume(StorageVolume):
     """
     Build and install xml for use on pools which use file based storage
     """
+    _file_type = VIR_STORAGE_VOL_FILE
 
     formats = ["raw", "bochs", "cloop", "cow", "dmg", "iso", "qcow",\
                "qcow2", "vmdk", "vpc"]
@@ -963,24 +1251,11 @@ class FileVolume(StorageVolume):
     def _get_source_xml(self):
         return ""
 
-#class DiskVolume(StorageVolume):
-#    """
-#    Build and install xml for use on disk device pools
-#    """
-#    def __init__(self, *args, **kwargs):
-#        raise NotImplementedError
-
-#class iSCSIVolume(StorageVolume):
-#    """
-#    Build and install xml for use on iSCSI device pools
-#    """
-#    def __init__(self, *args, **kwargs):
-#        raise NotImplementedError
-
-class LogicalVolume(StorageVolume):
+class DiskVolume(StorageVolume):
     """
-    Build and install logical volumes for lvm pools
+    Build and install xml volumes for use on physical disk pools
     """
+    _file_type = VIR_STORAGE_VOL_BLOCK
 
     # Register applicable property methods from parent class
     perms = property(StorageObject.get_perms, StorageObject.set_perms)
@@ -998,3 +1273,69 @@ class LogicalVolume(StorageVolume):
 
     def _get_source_xml(self):
         return ""
+
+class LogicalVolume(StorageVolume):
+    """
+    Build and install logical volumes for lvm pools
+    """
+    _file_type = VIR_STORAGE_VOL_BLOCK
+
+    # Register applicable property methods from parent class
+    perms = property(StorageObject.get_perms, StorageObject.set_perms)
+
+    def __init__(self, name, capacity, pool=None, pool_name=None, conn=None,
+                 allocation=None, perms=None):
+        StorageVolume.__init__(self, name=name, pool=pool, pool_name=pool_name,
+                               allocation=allocation, capacity=capacity,
+                               conn=conn)
+        if perms:
+            self.perms = perms
+
+    def _get_target_xml(self):
+        return "%s" % self._get_perms_xml()
+
+    def _get_source_xml(self):
+        return ""
+
+class CloneVolume(StorageVolume):
+    """
+    Build and install a volume that is a clone of an existing volume
+    """
+
+    format = property(StorageVolume.get_format, StorageVolume.set_format)
+
+    def __init__(self, name, input_vol):
+        if not isinstance(input_vol, libvirt.virStorageVol):
+            raise ValueError(_("input_vol must be a virStorageVol"))
+
+        pool = input_vol.storagePoolLookupByVolume()
+
+        # Populate some basic info
+        xml  = input_vol.XMLDesc(0)
+        typ  = input_vol.info()[0]
+        cap  = int(_util.get_xml_path(xml, "/volume/capacity"))
+        alc  = int(_util.get_xml_path(xml, "/volume/allocation"))
+        fmt  = _util.get_xml_path(xml, "/volume/target/format/@type")
+
+        StorageVolume.__init__(self, name=name, pool=pool,
+                               pool_name=pool.name(),
+                               allocation=alc, capacity=cap)
+
+        self.input_vol = input_vol
+        self._file_type = typ
+        self._format = fmt
+
+    def get_xml_config(self):
+        xml  = self.input_vol.XMLDesc(0)
+        newxml = _util.set_xml_path(xml, "/volume/name", self.name)
+        return newxml
+
+#class iSCSIVolume(StorageVolume):
+#    """
+#    Build and install xml for use on iSCSI device pools
+#    """
+#    _file_type = VIR_STORAGE_VOL_BLOCK
+#
+#    def __init__(self, *args, **kwargs):
+#        raise NotImplementedError
+
