@@ -19,9 +19,12 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 # MA 02110-1301 USA.
 
-import os, statvfs
+import os, stat, pwd, statvfs
 import subprocess
 import logging
+import re
+
+import urlgrabber.progress as progress
 import libvirt
 
 import _util
@@ -43,6 +46,71 @@ def _vdisk_create(path, size, kind, sparse = True):
     except OSError:
         return False
 
+def _vdisk_clone(path, clone):
+    logging.debug("Using vdisk clone.")
+
+    path = os.path.expanduser(path)
+    clone = os.path.expanduser(clone)
+    try:
+        rc = subprocess.call([ '/usr/sbin/vdiskadm', 'clone', path, clone ])
+        return rc == 0
+    except OSError:
+        return False
+
+def _qemu_sanitize_drvtype(phystype, fmt):
+    """
+    Sanitize libvirt storage volume format to a valid qemu driver type
+    """
+    raw_list = [ "iso" ]
+
+    if phystype == VirtualDisk.TYPE_BLOCK:
+        return VirtualDisk.DRIVER_QEMU_RAW
+
+    if fmt in raw_list:
+        return VirtualDisk.DRIVER_QEMU_RAW
+
+    return fmt
+
+def _name_uid(user):
+    """
+    Return UID for string username
+    """
+    pwdinfo = pwd.getpwnam(user)
+    return pwdinfo[2]
+
+def _is_dir_searchable(uid, username, path):
+    """
+    Check if passed directory is searchable by uid
+    """
+    try:
+        statinfo = os.stat(path)
+    except OSError:
+        return False
+
+    if uid == statinfo.st_uid:
+        flag = stat.S_IXUSR
+    elif uid == statinfo.st_gid:
+        flag = stat.S_IXGRP
+    else:
+        flag = stat.S_IXOTH
+
+    if bool(statinfo.st_mode & flag):
+        return True
+
+    # Check POSIX ACL (since that is what we use to 'fix' access)
+    cmd = ["getfacl", path]
+    proc = subprocess.Popen(cmd,
+                            stdout=subprocess.PIPE,
+                            stderr=subprocess.PIPE)
+    out, err = proc.communicate()
+
+    if proc.returncode != 0:
+        logging.debug("Cmd '%s' failed: %s" % (cmd, err))
+        return False
+
+    return bool(re.search("user:%s:..x" % username, out))
+
+
 class VirtualDisk(VirtualDevice):
     """
     Builds a libvirt domain disk xml description
@@ -74,10 +142,16 @@ class VirtualDisk(VirtualDevice):
     and will throw ValueError's if problems are found.
     """
 
+    _virtual_device_type = VirtualDevice.VIRTUAL_DEV_DISK
+
     DRIVER_FILE = "file"
     DRIVER_PHY = "phy"
     DRIVER_TAP = "tap"
-    driver_names = [DRIVER_FILE, DRIVER_PHY, DRIVER_TAP]
+    DRIVER_QEMU = "qemu"
+    driver_names = [DRIVER_FILE, DRIVER_PHY, DRIVER_TAP, DRIVER_QEMU]
+
+    DRIVER_QEMU_RAW = "raw"
+    # No list here, since there are many other valid values
 
     DRIVER_TAP_RAW = "aio"
     DRIVER_TAP_QCOW = "qcow"
@@ -101,11 +175,119 @@ class VirtualDisk(VirtualDevice):
     TYPE_BLOCK = "block"
     types = [TYPE_FILE, TYPE_BLOCK]
 
+    @staticmethod
+    def path_exists(conn, path):
+        """
+        Check if path exists. If we can't determine, return False
+        """
+        is_remote = _util.is_uri_remote(conn.getURI())
+        try:
+            vol = None
+            try:
+                vol = conn.storageVolLookupByPath(path)
+            except:
+                pass
+
+            if vol:
+                return True
+
+            if not is_remote:
+                return os.path.exists(path)
+        except:
+            pass
+
+        return False
+
+    @staticmethod
+    def check_path_search_for_user(conn, path, username):
+        """
+        Check if the passed user has search permissions for all the
+        directories in the disk path.
+
+        @return: List of the directories the user cannot search, or empty list
+        @rtype : C{list}
+        """
+        if _util.is_uri_remote(conn.getURI()):
+            return []
+
+        try:
+            uid = _name_uid(username)
+        except Exception, e:
+            logging.debug("Error looking up username: %s" % str(e))
+            return []
+
+        fixlist = []
+
+        if os.path.isdir(path):
+            dirname = path
+            base = "-"
+        else:
+            dirname, base = os.path.split(path)
+
+        while base:
+            if not _is_dir_searchable(uid, username, dirname):
+                fixlist.append(dirname)
+
+            dirname, base = os.path.split(dirname)
+
+        return fixlist
+
+    @staticmethod
+    def fix_path_search_for_user(conn, path, username):
+        """
+        Try to fix any permission problems found by check_path_search_for_user
+
+        @return: Return a dictionary of entries { broken path : error msg }
+        @rtype : C{dict}
+        """
+        def fix_perms(dirname, useacl=True):
+            if useacl:
+                cmd = ["setfacl", "--modify", "user:%s:x" % username, dirname]
+                proc = subprocess.Popen(cmd,
+                                        stdout=subprocess.PIPE,
+                                        stderr=subprocess.PIPE)
+                out, err = proc.communicate()
+
+                logging.debug("Ran command '%s'" % cmd)
+                if out or err:
+                    logging.debug("out=%s\nerr=%s" % (out, err))
+
+                if proc.returncode != 0:
+                    raise ValueError(err)
+            else:
+                mode = os.stat(dirname).st_mode
+                os.chmod(dirname, mode | stat.S_IXOTH)
+
+        fixlist = VirtualDisk.check_path_search_for_user(conn, path, username)
+        if not fixlist:
+            return []
+
+        fixlist.reverse()
+        errdict = {}
+
+        useacl = True
+        for dirname in fixlist:
+            try:
+                try:
+                    fix_perms(dirname, useacl)
+                except:
+                    # If acl fails, fall back to chmod and retry
+                    if not useacl:
+                        raise
+                    useacl = False
+
+                    fix_perms(dirname, useacl)
+            except Exception, e:
+                errdict[dirname] =  str(e)
+
+        return errdict
+
+
     def __init__(self, path=None, size=None, transient=False, type=None,
                  device=DEVICE_DISK, driverName=None, driverType=None,
                  readOnly=False, sparse=True, conn=None, volObject=None,
                  volInstall=None, volName=None, bus=None, shareable=False,
-                 driverCache=None):
+                 driverCache=None, selinuxLabel=None, format=None):
         """
         @param path: filesystem path to the disk image.
         @type path: C{str}
@@ -140,6 +322,10 @@ class VirtualDisk(VirtualDevice):
         @type shareable: C{bool}
         @param driverCache: Disk cache mode (none, writethrough, writeback)
         @type driverCache: member of cache_types
+        @param selinuxLabel: Used for labelling new or relabel existing storage
+        @type selinuxLabel: C{str}
+        @param format: Storage volume format to use when creating storage
+        @type format: C{str}
         """
 
         VirtualDevice.__init__(self, conn=conn)
@@ -155,6 +341,9 @@ class VirtualDisk(VirtualDevice):
         self._bus = None
         self._shareable = None
         self._driver_cache = None
+        self._selinux_label = None
+        self._clone_path = None
+        self._format = None
 
         # XXX: No property methods for these
         self.transient = transient
@@ -173,6 +362,8 @@ class VirtualDisk(VirtualDevice):
         self._set_bus(bus, validate=False)
         self._set_shareable(shareable, validate=False)
         self._set_driver_cache(driverCache, validate=False)
+        self._set_selinux_label(selinuxLabel, validate=False)
+        self._set_format(format, validate=False)
 
         if volName:
             self.__lookup_vol_name(volName)
@@ -197,6 +388,26 @@ class VirtualDisk(VirtualDevice):
         self.__validate_wrapper("_path", val, validate)
     path = property(_get_path, _set_path)
 
+    def _get_clone_path(self):
+        return self._clone_path
+    def _set_clone_path(self, val, validate=True):
+        if val is not None:
+            self._check_str(val, "path")
+            val = os.path.abspath(val)
+
+            # Pass the path to a VirtualDisk, which should provide validation
+            # for us
+            try:
+                # If this disk isn't managed, don't pass 'conn' to this
+                # validation disk, to ensure we have permissions for manual
+                # cloning
+                conn = self.__storage_specified() and self.conn or None
+                VirtualDisk(conn=conn, path=val)
+            except Exception, e:
+                raise ValueError(_("Error validating clone path: %s") % e)
+        self.__validate_wrapper("_clone_path", val, validate)
+    clone_path = property(_get_clone_path, _set_clone_path)
+
     def _get_size(self):
         return self._size
     def _set_size(self, val, validate=True):
@@ -293,6 +504,30 @@ class VirtualDisk(VirtualDevice):
         self.__validate_wrapper("_driver_cache", val, validate)
     driver_cache = property(_get_driver_cache, _set_driver_cache)
 
+    # If there is no selinux support on the libvirt connection or the
+    # system, we won't throw errors if this is set, just silently ignore.
+    def _get_selinux_label(self):
+        return self._selinux_label
+    def _set_selinux_label(self, val, validate=True):
+        if val is not None:
+            self._check_str(val, "selinux_label")
+
+            if (self._support_selinux() and
+                not _util.selinux_is_label_valid(val)):
+                # XXX Not valid if we support changing labels remotely
+                raise ValueError(_("SELinux label '%s' is not valid.") % val)
+
+        self.__validate_wrapper("_selinux_label", val, validate)
+    selinux_label = property(_get_selinux_label, _set_selinux_label)
+
+    def _get_format(self):
+        return self._format
+    def _set_format(self, val, validate=True):
+        if val is not None:
+            self._check_str(val, "format")
+        self.__validate_wrapper("_format", val, validate)
+    format = property(_get_format, _set_format)
+
     # Validation assistance methods
 
     # Initializes attribute if it hasn't been done, then validates args.
@@ -310,12 +545,30 @@ class VirtualDisk(VirtualDevice):
                 setattr(self, varname, orig)
                 raise
 
-    def __set_size(self, creating_storage):
+    def __set_format(self):
+        if not self.format:
+            return
+
+        if not self.__creating_storage():
+            return
+
+        if self.vol_install:
+            if not hasattr(self.vol_install, "format"):
+                raise ValueError(_("Storage type does not support format "
+                                   "parameter."))
+            if self.vol_install.format != self.format:
+                self.vol_install.format = self.format
+
+        elif self.format != "raw":
+            raise RuntimeError(_("Format cannot be specified for "
+                                 "unmanaged storage."))
+
+    def __set_size(self):
         """
         Fill in 'size' attribute for existing storage.
         """
 
-        if creating_storage:
+        if self.__creating_storage():
             return
 
         if self.__storage_specified() and self.vol_object:
@@ -332,8 +585,6 @@ class VirtualDisk(VirtualDevice):
             newsize = newsize / 1024.0 / 1024.0 / 1024.0
 
         if newsize != self.size:
-            logging.debug("Setting size for existing storage to '%s'" %
-                          newsize)
             self._set_size(newsize, validate=False)
 
     def __set_dev_type(self):
@@ -352,27 +603,70 @@ class VirtualDisk(VirtualDevice):
             else:
                 raise ValueError, _("Unknown storage volume type.")
         elif self.vol_install:
-            if isinstance(self.vol_install, Storage.FileVolume):
+            if self.vol_install.file_type == libvirt.VIR_STORAGE_VOL_FILE:
                 dtype = self.TYPE_FILE
             else:
-                # All others should be using TYPE_BLOCK (hopefully)
                 dtype = self.TYPE_BLOCK
         elif self.path:
             if _util.stat_disk(self.path)[0]:
                 dtype = self.TYPE_FILE
             else:
                 dtype = self.TYPE_BLOCK
-            if _util.is_vdisk(self.path):
-                self._driverName = self.DRIVER_TAP
-                self._driverType = self.DRIVER_TAP_VDISK
 
-        if self.type is None:
-            logging.debug("Detected storage as type '%s'" % dtype)
-        elif dtype != self.type:
+        if not dtype:
+            dtype = self.type or self.TYPE_BLOCK
+
+        elif self.type and dtype != self.type:
             raise ValueError(_("Passed type '%s' does not match detected "
                                "storage type '%s'" % (self.type, dtype)))
         self.set_type(dtype, validate=False)
 
+    def __set_driver(self):
+        """
+        Set driverName and driverType from passed parameters
+
+        Where possible, we want to force driverName = "raw" if installing
+        a QEMU VM. Without telling QEMU to expect a raw file, the emulator
+        is forced to autodetect, which has security implications:
+
+        http://lists.gnu.org/archive/html/qemu-devel/2008-04/msg00675.html
+        """
+        drvname = self._driverName
+        drvtype = self._driverType
+
+        if self.conn:
+            driver = _util.get_uri_driver(self._get_uri())
+            if driver.lower() == "qemu":
+                drvname = self.DRIVER_QEMU
+
+        if self.vol_object:
+            fmt = _util.get_xml_path(self.vol_object.XMLDesc(0),
+                                     "/volume/target/format/@type")
+            if drvname == self.DRIVER_QEMU:
+                drvtype = _qemu_sanitize_drvtype(self.type, fmt)
+
+        elif self.vol_install:
+            if drvname == self.DRIVER_QEMU:
+                if hasattr(self.vol_install, "format"):
+                    drvtype = _qemu_sanitize_drvtype(self.type,
+                                                     self.vol_install.format)
+
+        elif self.__creating_storage():
+            if drvname == self.DRIVER_QEMU:
+                drvtype = self.DRIVER_QEMU_RAW
+
+        elif self.path and os.path.exists(self.path):
+            if _util.is_vdisk(self.path):
+                drvname = self.DRIVER_TAP
+                drvtype = self.DRIVER_TAP_VDISK
+
+        if self._driverName and self._driverName != drvname:
+            # User already set driverName to a different value, respect that
+            return
+
+        self._driverName = drvname or None
+        self._driverType = drvtype or None
+
     def __lookup_vol_name(self, name_tuple):
         """
         lookup volume via tuple passed via __init__'s volName parameter
@@ -399,6 +693,26 @@ class VirtualDisk(VirtualDevice):
         """
         return (self.vol_object != None or self.vol_install != None)
 
+    def __creating_storage(self):
+        """
+        Return True if the user requested us to create a device
+        """
+        return not (self.__no_storage() or
+                    (self.__storage_specified() and self.vol_object) or
+                    (self.path and os.path.exists(self.path)))
+
+    def __no_storage(self):
+        """
+        Return True if no path or storage was specified
+        """
+        no_storage = (not self.__storage_specified() and not self.path)
+        if no_storage:
+            if (self.device != self.DEVICE_FLOPPY and
+                self.device != self.DEVICE_CDROM):
+                raise ValueError(_("Device type '%s' requires a path") %
+                                 self.device)
+        return no_storage
+
     def __check_if_path_managed(self):
         """
         Determine if we can use libvirt storage apis to create or lookup
@@ -415,7 +729,10 @@ class VirtualDisk(VirtualDevice):
 
             try:
                 vol = self.conn.storageVolLookupByPath(self.path)
+                vol.info()
             except Exception, e:
+                vol = None
+
                 try:
                     try:
                         # Pool may need to be refreshed, but if it errors,
@@ -425,7 +742,9 @@ class VirtualDisk(VirtualDevice):
                         pool = None
                         raise
                     vol = self.conn.storageVolLookupByPath(self.path)
+                    vol.info()
                 except Exception, e:
+                    vol = None
                     verr = str(e)
 
         if not vol:
@@ -445,10 +764,7 @@ class VirtualDisk(VirtualDevice):
                 if self.sparse:
                     alloc = 0
                 else:
-                    #alloc = cap
-                    # XXX: disable setting managed storage as nonsparse
-                    # XXX: since it hoses libvirtd (for now)
-                    alloc = 0
+                    alloc = cap
                 vol = volclass(name=os.path.basename(self.path),
                                capacity=cap, allocation=alloc, pool=pool)
                 self._set_vol_install(vol, validate=False)
@@ -495,11 +811,29 @@ class VirtualDisk(VirtualDevice):
                               "StorageVolume object")
                 self._set_size(newsize, validate=False)
 
-        # Remove this piece when storage volume creation is async
-        if self.sparse and self.vol_install and \
-           self.vol_install.allocation != 0:
-            logging.debug("Setting vol_install allocation to 0 (sparse).")
-            self.vol_install.allocation = 0
+    def _storage_security_label(self):
+        """
+        Return SELinux label of existing storage, or None
+        """
+        context = ""
+
+        if self.__no_storage():
+            return context
+
+        if self.vol_object:
+            context = _util.get_xml_path(self.vol_object.XMLDesc(0),
+                                         "/volume/target/permissions/label")
+        elif self.vol_install:
+            # XXX: If user entered a manual label, should we sync this
+            # to vol_install?
+            l = _util.get_xml_path(self.vol_install.pool.XMLDesc(0),
+                                   "/pool/target/permissions/label")
+            context = l or ""
+        else:
+            context = _util.selinux_getfilecon(self.path)
+
+        return context
+
 
     def __validate_params(self):
         """
@@ -507,47 +841,53 @@ class VirtualDisk(VirtualDevice):
         disk parameters.
         """
 
-        # if storage capable, try to lookup path
-        # if no obj: if remote, error
-        storage_capable = False
-        if self.conn:
-            storage_capable = _util.is_storage_capable(self.conn)
-
-        if not storage_capable and self._is_remote():
-            raise ValueError, _("Connection doesn't support remote storage.")
-
-        # If the user didn't pass storage parameters, try to determine them
-        # from the passed path
-        if storage_capable and self.path is not None \
-           and not self.__storage_specified():
-            self.__check_if_path_managed()
-
-        # Sync parameters between VirtualDisk and potentially passed
-        # storage objects.
-        self.__sync_params()
-
-        # One small caveat: if self.path isn't set at this point, we are
-        # basically done.
-        if self.path is None:
-            if self.device != self.DEVICE_FLOPPY and \
-               self.device != self.DEVICE_CDROM:
-                raise ValueError, _("Device type '%s' requires a path") % \
-                                  self.device
+        if self.__no_storage():
+            # No storage specified for a removable device type (CDROM, floppy)
+            self._type = self.TYPE_BLOCK
             return True
 
+        storage_capable = bool(self.conn and
+                               _util.is_storage_capable(self.conn))
+
+        if storage_capable and not self.__storage_specified():
+            # Try to lookup self.path storage objects
+            self.__check_if_path_managed()
+
+        if self._is_remote():
+            if not storage_capable:
+                raise ValueError, _("Connection doesn't support remote "
+                                    "storage.")
+            if not self.__storage_specified():
+                raise ValueError, _("Must specify libvirt managed storage "
+                                    "if on a remote connection")
+
+        # Sync parameters between VirtualDisk and  storage objects.
+        self.__sync_params()
 
         # The main distinctions from this point forward:
-        # Are we doing storage API operations or local media checks?
-        managed_storage = self.__storage_specified() or self.path is None
-        # Do we need to create the storage?
-        create_media = not ((managed_storage and self.vol_object) or \
-                            (self.path and os.path.exists(self.path)))
+        # - Are we doing storage API operations or local media checks?
+        # - Do we need to create the storage?
 
-        self.__set_size(create_media)
+        managed_storage = self.__storage_specified()
+        create_media = self.__creating_storage()
 
-        if self._is_remote() and not managed_storage:
-            raise ValueError, _("Must specify libvirt managed storage if on "
-                                "a remote connection")
+        self.__set_dev_type()
+        self.__set_size()
+        self.__set_format()
+        self.__set_driver()
+
+        if not self.selinux_label:
+            # If we are using existing storage, pull the label from it
+            # If we are installing via vol_install, pull from the parent pool
+            # If we are creating local storage, use the expected label
+            context = ""
+
+            if create_media and not managed_storage:
+                context = self._expected_security_label()
+            else:
+                context = self._storage_security_label()
+
+            self._selinux_label = context or ""
 
         # If not creating the storage, our job is easy
         if not create_media:
@@ -557,9 +897,7 @@ class VirtualDisk(VirtualDevice):
                     # vdisk _is_ a directory.
                     raise ValueError(_("The path '%s' must be a file or a "
                                        "device, not a directory") % self.path)
-                # XXX: Any selinux validation checks should go here
 
-            self.__set_dev_type()
             return True
 
 
@@ -570,8 +908,8 @@ class VirtualDisk(VirtualDevice):
 
         if not managed_storage:
             if self.type is self.TYPE_BLOCK:
-                raise ValueError, _("Local block device path must exist.")
-            self.set_type(self.TYPE_FILE, validate=False)
+                raise ValueError, _("Local block device path '%s' must "
+                                    "exist.") % self.path
 
             # Path doesn't exist: make sure we have write access to dir
             if not os.access(os.path.dirname(self.path), os.R_OK):
@@ -583,8 +921,6 @@ class VirtualDisk(VirtualDevice):
             if not os.access(os.path.dirname(self.path), os.W_OK):
                 raise ValueError, _("No write access to directory '%s'") % \
                                     os.path.dirname(self.path)
-        else:
-            self.__set_dev_type()
 
         # Applicable for managed or local storage
         ret = self.is_size_conflict()
@@ -593,7 +929,135 @@ class VirtualDisk(VirtualDevice):
         elif ret[1]:
             logging.warn(ret[1])
 
+    # Storage creation routines
+    def _do_create_storage(self, progresscb):
+        # If a clone_path is specified, but not vol_install.input_vol,
+        # that means we are cloning unmanaged -> managed, so skip this
+        if (self.vol_install and
+            (not self.clone_path or self.vol_install.input_vol)):
+            self._set_vol_object(self.vol_install.install(meter=progresscb),
+                                 validate=False)
+            # Then just leave: vol_install should handle any selinux stuff
+            return
 
+        if self.clone_path:
+            text = (_("Cloning %(srcfile)s") %
+                    {'srcfile' : os.path.basename(self.clone_path)})
+        else:
+            text=_("Creating storage file %s") % os.path.basename(self.path)
+
+        size_bytes = long(self.size * 1024L * 1024L * 1024L)
+        progresscb.start(filename=self.path, size=long(size_bytes),
+                         text=text)
+
+        if self.clone_path:
+            # VDisk clone
+            if (_util.is_vdisk(self.clone_path) or
+                (os.path.exists(self.path) and _util.is_vdisk(self.path))):
+
+                if (not _util.is_vdisk(self.clone_path) or
+                    os.path.exists(self.path)):
+                    raise RuntimeError, _("copying to an existing vdisk is not"
+                                          " supported")
+                if not _vdisk_clone(self.clone_path, self.path):
+                    raise RuntimeError, _("failed to clone disk")
+                progresscb.end(size_bytes)
+
+            else:
+                # Plain file clone
+                self._clone_local(progresscb, size_bytes)
+
+        elif _util.is_vdisk(self.path):
+            # Create vdisk
+            progresscb.update(1024)
+            if not _vdisk_create(self.path, size_bytes, "vmdk", self.sparse):
+                raise RuntimeError, _("Error creating vdisk %s" % self.path)
+
+            progresscb.end(self.size)
+        else:
+            # Plain file creation
+            self._create_local_file(progresscb, size_bytes)
+
+    def _create_local_file(self, progresscb, size_bytes):
+        """
+        Helper function which attempts to build self.path
+        """
+        fd = None
+
+        try:
+            try:
+                fd = os.open(self.path, os.O_WRONLY | os.O_CREAT)
+                if self.sparse:
+                    os.ftruncate(fd, size_bytes)
+                    progresscb.update(self.size)
+                else:
+                    buf = '\x00' * 1024 * 1024 # 1 meg of nulls
+                    for i in range(0, long(self.size * 1024L)):
+                        os.write(fd, buf)
+                        progresscb.update(long(i * 1024L * 1024L))
+            except OSError, e:
+                raise RuntimeError(_("Error creating diskimage %s: %s") %
+                                   (self.path, str(e)))
+        finally:
+            if fd is not None:
+                os.close(fd)
+            progresscb.end(size_bytes)
+
+    def _clone_local(self, meter, size_bytes):
+
+        # if a destination file exists and sparse flg is True,
+        # this priority takes a existing file.
+        if (os.path.exists(self.path) == False and self.sparse == True):
+            clone_block_size = 4096
+            sparse = True
+            fd = None
+            try:
+                fd = os.open(self.path, os.O_WRONLY | os.O_CREAT)
+                os.ftruncate(fd, size_bytes)
+            finally:
+                if fd:
+                    os.close(fd)
+        else:
+            clone_block_size = 1024*1024*10
+            sparse = False
+
+        logging.debug("Local Cloning %s to %s, sparse=%s, block_size=%s" %
+                      (self.clone_path, self.path, sparse, clone_block_size))
+
+        zeros = '\0' * 4096
+
+        src_fd, dst_fd = None, None
+        try:
+            try:
+                src_fd = os.open(self.clone_path, os.O_RDONLY)
+                dst_fd = os.open(self.path, os.O_WRONLY | os.O_CREAT)
+
+                i=0
+                while 1:
+                    l = os.read(src_fd, clone_block_size)
+                    s = len(l)
+                    if s == 0:
+                        meter.end(size_bytes)
+                        break
+                    # check sequence of zeros
+                    if sparse and zeros == l:
+                        os.lseek(dst_fd, s, 1)
+                    else:
+                        b = os.write(dst_fd, l)
+                        if s != b:
+                            meter.end(i)
+                            break
+                    i += s
+                    if i < size_bytes:
+                        meter.update(i)
+            except OSError, e:
+                raise RuntimeError(_("Error cloning diskimage %s to %s: %s") %
+                                       (self.clone_path, self.path, str(e)))
+        finally:
+            if src_fd is not None:
+                os.close(src_fd)
+            if dst_fd is not None:
+                os.close(dst_fd)
 
     def setup(self, progresscb=None):
         """
@@ -605,53 +1069,23 @@ class VirtualDisk(VirtualDevice):
         @param progresscb: progress meter
         @type progresscb: instanceof urlgrabber.BaseMeter
         """
-        if self.vol_object:
-            return
-        elif self.vol_install:
-            self._set_vol_object(self.vol_install.install(meter=progresscb),
-                                 validate=False)
-            return
-        elif (self.type == VirtualDisk.TYPE_FILE and self.path is not None
-             and not os.path.exists(self.path)):
-            size_bytes = long(self.size * 1024L * 1024L * 1024L)
+        if not progresscb:
+            progresscb = progress.BaseMeter()
 
-            if progresscb:
-                progresscb.start(filename=self.path,size=long(size_bytes), \
-                                 text=_("Creating storage file..."))
+        if self.__creating_storage() or self.clone_path:
+            self._do_create_storage(progresscb)
 
-            if _util.is_vdisk(self.path):
-                progresscb.update(1024)
-                if (not _vdisk_create(self.path, size_bytes, "vmdk",
-                    self.sparse)):
-                    raise RuntimeError, _("Error creating vdisk %s" % self.path)
-                self._driverName = self.DRIVER_TAP
-                self._driverType = self.DRIVER_TAP_VDISK
-                progresscb.end(self.size)
-                return
-
-            fd = None
-            try:
-                try:
-                    fd = os.open(self.path, os.O_WRONLY | os.O_CREAT)
-                    if self.sparse:
-                        os.ftruncate(fd, size_bytes)
-                        if progresscb:
-                            progresscb.update(self.size)
-                    else:
-                        buf = '\x00' * 1024 * 1024 # 1 meg of nulls
-                        for i in range(0, long(self.size * 1024L)):
-                            os.write(fd, buf)
-                            if progresscb:
-                                progresscb.update(long(i * 1024L * 1024L))
-                except OSError, e:
-                    raise RuntimeError, _("Error creating diskimage %s: %s" % \
-                                        (self.path, str(e)))
-            finally:
-                if fd is not None:
-                    os.close(fd)
-                if progresscb:
-                    progresscb.end(size_bytes)
-        # FIXME: set selinux context?
+        # Relabel storage if it was requested
+        storage_label = self._storage_security_label()
+        if storage_label and storage_label != self.selinux_label:
+            if not self._support_selinux():
+                logging.debug("No support for changing selinux context.")
+            elif not self._security_can_fix():
+                logging.debug("Can't fix selinux context in this case.")
+            else:
+                logging.debug("Changing path=%s selinux label %s -> %s" %
+                              (self.path, storage_label, self.selinux_label))
+                _util.selinux_setfilecon(self.path, self.selinux_label)
 
     def get_xml_config(self, disknode=None):
         """
@@ -727,8 +1161,7 @@ class VirtualDisk(VirtualDevice):
         if self.vol_install:
             return self.vol_install.is_size_conflict()
 
-        if self.vol_object or self.size is None or not self.path \
-           or os.path.exists(self.path) or self.type != self.TYPE_FILE:
+        if not self.__creating_storage():
             return (False, None)
 
         ret = False
@@ -801,22 +1234,97 @@ class VirtualDisk(VirtualDevice):
 
         return ret
 
+    def _support_selinux(self):
+        """
+        Return True if we have the requisite libvirt and library support
+        for selinux commands
+        """
+        if (not self._caps and False):
+            #self._caps.host.secmodel is None or
+            #self._caps.host.secmodel.model != "selinux"):
+            # XXX: Libvirt support isn't strictly required, but all the
+            #      our label guesses are built with svirt in mind
+            return False
+
+        elif self._is_remote():
+            return False
+
+        elif not _util.have_selinux():
+            # XXX: When libvirt supports changing labels via storage APIs,
+            #      this will need changing.
+            return False
+
+        elif self.__storage_specified() and self.path:
+            try:
+                statinfo = os.stat(self.path)
+            except:
+                return False
+
+            # Not sure if this is even the correct metric for
+            # 'Can we change the file context'
+            return os.geteuid() in ['0', statinfo.st_uid]
+
+        return True
+
+    def _expected_security_label(self):
+        """
+        Best guess at what the expected selinux label should be for the disk
+        """
+        label = None
+
+        # XXX: These are really only approximations in the remote case?
+        # XXX: Maybe libvirt should expose the relevant selinux labels in
+        #      the capabilities XML?
+
+        if not self._support_selinux():
+            pass
+        elif self.__no_storage():
+            pass
+        elif self.read_only:
+            label = _util.selinux_readonly_label()
+        elif self.shareable:
+            # XXX: Should this be different? or do we not care about MLS here?
+            label = _util.selinux_rw_label()
+        else:
+            label = _util.selinux_rw_label()
+
+        return label or ""
+
+    def _security_can_fix(self):
+        can_fix = True
+
+        if not self._support_selinux():
+            can_fix = False
+        elif self.__no_storage():
+            can_fix = False
+        elif self.type == VirtualDisk.TYPE_BLOCK:
+            # Shouldn't change labelling on block devices (though we can)
+            can_fix = False
+        elif not self.read_only:
+            # XXX Leave all other (R/W disk) relabeling up to libvirt/svirt
+            # for now
+            can_fix = False
+
+        return can_fix
+
     def _get_target_type(self):
         """
         Returns the suggested disk target prefix (hd, xvd, sd ...) from
         the passed parameters.
         @returns: str prefix, or None if no reasonable guess can be made
         """
+        # The upper limits here aren't necessarilly 1024, but let the HV
+        # error as appropriate.
         if self.bus == "virtio":
-            return ("vd", 16)
+            return ("vd", 1024)
         elif self.bus == "scsi" or self.bus == "usb":
-            return ("sd", 16)
+            return ("sd", 1024)
         elif self.bus == "xen":
-            return ("xvd", 16)
+            return ("xvd", 1024)
+        elif self.bus == "fdc" or self.device == self.DEVICE_FLOPPY:
+            return ("fd", 2)
         elif self.bus == "ide":
             return ("hd", 4)
-        elif self.bus == "floppy" or self.device == self.DEVICE_FLOPPY:
-            return ("fd", 2)
         else:
             return (None, None)
 
