@@ -20,10 +20,14 @@
 import VirtualDevice
 import NodeDeviceParser
 import logging
+
+from virtinst import _util
 from virtinst import _virtinst as _
 
 class VirtualHostDevice(VirtualDevice.VirtualDevice):
 
+    _virtual_device_type = VirtualDevice.VirtualDevice.VIRTUAL_DEV_HOSTDEV
+
     def device_from_node(conn, name=None, nodedev=None):
         """
         Convert the passed libvirt node device name to a VirtualHostDevice
@@ -74,6 +78,8 @@ class VirtualHostDevice(VirtualDevice.VirtualDevice):
         self.type = None
 
         self.managed = True
+        if _util.get_uri_driver(self.conn.getURI()).lower() == "xen":
+            self.managed = False
 
         self._nodedev = nodedev
 
@@ -95,7 +101,7 @@ class VirtualHostDevice(VirtualDevice.VirtualDevice):
         xml += "      <source>\n"
         xml += self._get_source_xml()
         xml += "      </source>\n"
-        xml += "    </hostdev>\n"
+        xml += "    </hostdev>"
         return xml
 
 
@@ -200,9 +206,11 @@ class VirtualHostDevicePCI(VirtualHostDevice):
 
         try:
             # Do this as a sanity check, so that we don't fail at domain
-            # start time
-            self._nodedev.deviceDetach()
-            self._nodedev.deviceReset()
+            # start time. This is independent of the 'managed' state, since
+            # this should work regardless.
+            node = conn.nodeDeviceLookupByName(self._nodedev.name)
+            node.dettach()
+            node.reset()
         except Exception, e:
             raise RuntimeError(_("Could not detach PCI device: %s" % str(e)))
 
