@@ -24,16 +24,22 @@ gettext.bindtextdomain(gettext_app, gettext_dir)
 def _virtinst(msg):
     return gettext.dgettext(gettext_app, msg)
 
-import util
+from version import __version__
+__version_info__ = tuple([ int(num) for num in __version__.split('.')])
+
 import Storage
 from Guest import Guest, XenGuest
+from VirtualDevice import VirtualDevice
 from VirtualNetworkInterface import VirtualNetworkInterface, \
                                     XenNetworkInterface
 from VirtualGraphics import VirtualGraphics
 from VirtualAudio import VirtualAudio
+from VirtualInputDevice import VirtualInputDevice
 from VirtualDisk import VirtualDisk, XenDisk
 from VirtualHostDevice import (VirtualHostDevice, VirtualHostDeviceUSB,
                                VirtualHostDevicePCI)
+from VirtualCharDevice import VirtualCharDevice
+from VirtualVideoDevice import VirtualVideoDevice
 from FullVirtGuest import FullVirtGuest
 from ParaVirtGuest import ParaVirtGuest
 from DistroInstaller import DistroInstaller
@@ -43,6 +49,8 @@ from ImportInstaller import ImportInstaller
 from ImageManager import ImageInstaller
 from CloneManager import CloneDesign
 from User import User
+import util
+import support
 
 # This represents the PUBLIC API. Any changes to these classes (or 'util.py')
 # must be mindful of this fact.
@@ -51,4 +59,6 @@ __all__ = ["Guest", "XenGuest", "VirtualNetworkInterface",
            "VirtualDisk", "XenDisk", "FullVirtGuest", "ParaVirtGuest",
            "DistroInstaller", "PXEInstaller", "LiveCDInstaller",
            "ImportInstaller", "ImageInstaller", "CloneDesign", "Storage",
-           "User", "util"]
+           "User", "util", "support", "VirtualDevice",
+           "VirtualHostDevice", "VirtualHostDeviceUSB", "VirtualVideoDevice",
+           "VirtualHostDevicePCI", "VirtualCharDevice", "VirtualInputDevice"]
