@@ -19,6 +19,10 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 # MA 02110-1301 USA.
 
+import libvirt
+
+import _util
+from virtinst import _virtinst as _
 
 """
 Default values for OS_TYPES keys. Can be overwritten at os_type or
@@ -35,10 +39,26 @@ DEFAULTS = { \
     "devices" : {
      #  "devname" : { "attribute" : [( ["applicable", "hv-type", list"],
      #                               "recommended value for hv-types" ),]},
-        "input"   : { "type" : [ (["all"], "mouse") ],
-                      "bus"  : [ (["all"], "ps2") ] },
-        "disk"    : { "bus"  : [ (["all"], None) ] },
-        "net"     : { "model": [ (["all"], None) ] },
+        "input"   : {
+            "type" : [
+                (["all"], "mouse")
+            ],
+            "bus"  : [
+                (["all"], "ps2")
+            ],
+        },
+
+        "disk"    : {
+            "bus"  : [
+                (["all"], None)
+            ],
+        },
+
+        "net"     : {
+            "model": [
+                (["all"], None)
+            ],
+        },
     }
 }
 
@@ -64,12 +84,99 @@ def sort_helper(tosort):
 
     return retlist
 
+def parse_key_entry(conn, hv_type, key_entry):
+    d = _util.get_uri_driver(conn.getURI())
+    libver = libvirt.getVersion()
+    try:
+        drvver = libvirt.getVersion(d)[1]
+    except:
+        drvver = 0
+
+    ret = None
+    if type(key_entry) == list:
+        # List of tuples with hv_type, version, etc. mappings
+        for tup in key_entry:
+            exp_hvs = tup[0]
+            if type(exp_hvs) != list:
+                exp_hvs = [exp_hvs]
+            exp_hv_ver = 0
+            exp_lib_ver = 0
+            val = tup[-1]
+
+            if len(tup) > 2:
+                exp_hv_ver = tup[1]
+            if len(tup) > 3:
+                exp_lib_ver = tup[2]
+
+            if hv_type not in exp_hvs and "all" not in exp_hvs:
+                continue
+
+            if exp_hv_ver and drvver > exp_hv_ver:
+                continue
+
+            if exp_lib_ver and libver > exp_lib_ver:
+                continue
+
+            ret = val
+            break
+    else:
+        ret = key_entry
+
+    return ret
+
+def lookup_osdict_key(conn, hv_type, os_type, var, key):
+
+    dictval = DEFAULTS[key]
+    if os_type:
+        if var and OS_TYPES[os_type]["variants"][var].has_key(key):
+            dictval = OS_TYPES[os_type]["variants"][var][key]
+        elif OS_TYPES[os_type].has_key(key):
+            dictval = OS_TYPES[os_type][key]
+
+    return parse_key_entry(conn, hv_type, dictval)
+
+
+def lookup_device_param(conn, hv_type, os_type, var, device_key, param):
+
+    os_devs = lookup_osdict_key(conn, hv_type, os_type, var, "devices")
+    default_devs = DEFAULTS["devices"]
+
+    for devs in [os_devs, default_devs]:
+        if not devs.has_key(device_key):
+            continue
+
+        return parse_key_entry(conn, hv_type, devs[device_key][param])
+
+    raise RuntimeError(_("Invalid dictionary entry for device '%s %s'" %
+                       (device_key, param)))
+
+VIRTIO_DISK = {
+    "bus" : [
+        (["kvm"], "virtio"),
+    ]
+}
+
+VIRTIO_NET = {
+    "model" : [
+        (["kvm"], "virtio"),
+    ]
+}
+
+USB_TABLET = {
+    "type" : [
+        (["all"], "tablet"),
+    ],
+    "bus"  : [
+        (["all"], "usb"),
+    ]
+}
+
 # NOTE: keep variant keys using only lowercase so we can do case
 #       insensitive checks on user passed input
-OS_TYPES = {\
-"linux": { \
+OS_TYPES = {
+"linux": {
     "label": "Linux",
-    "variants": { \
+    "variants": {
         "rhel2.1": { "label": "Red Hat Enterprise Linux 2.1",
                      "distro": "rhel" },
         "rhel3": { "label": "Red Hat Enterprise Linux 3",
@@ -78,6 +185,18 @@ OS_TYPES = {\
                    "distro": "rhel" },
         "rhel5": { "label": "Red Hat Enterprise Linux 5",
                    "distro": "rhel" },
+        "rhel5.4": { "label": "Red Hat Enterprise Linux 5.4 or later",
+                     "distro": "rhel",
+                      "devices" : {
+                        "disk" : VIRTIO_DISK,
+                        "net"  : VIRTIO_NET,
+                      },},
+        "rhel6": { "label": "Red Hat Enterprise Linux 6", "distro": "rhel",
+                   "devices" : {
+                        "disk" : VIRTIO_DISK,
+                        "net"  : VIRTIO_NET,
+                        "input": USB_TABLET,
+                   }},
         "fedora5": { "sortby": "fedora05",
                      "label": "Fedora Core 5", "distro": "fedora" },
         "fedora6": { "sortby": "fedora06",
@@ -92,74 +211,110 @@ OS_TYPES = {\
                         # Apparently F9 has selinux errors when installing
                         # with virtio:
                         # https://bugzilla.redhat.com/show_bug.cgi?id=470386
-                        #"disk" : { "bus"   : [ (["kvm"], "virtio") ] },
-                        "net"  : { "model" : [ (["kvm"], "virtio") ] }
+                        #"disk" : VIRTIO_DISK,
+                        "net"  : VIRTIO_NET,
                       }},
         "fedora10": { "label": "Fedora 10", "distro": "fedora",
                       "devices" : {
-                        "disk" : { "bus"   : [ (["kvm"], "virtio") ] },
-                        "net"  : { "model" : [ (["kvm"], "virtio") ] }
+                        "disk" : VIRTIO_DISK,
+                        "net"  : VIRTIO_NET,
                       }},
         "fedora11": { "label": "Fedora 11", "distro": "fedora",
                       "devices" : {
-                        "disk" : { "bus"   : [ (["kvm"], "virtio") ] },
-                        "net"  : { "model" : [ (["kvm"], "virtio") ] },
-                        "input" : { "type" : [ (["all"], "tablet") ],
-                                    "bus"  : [ (["all"], "usb"), ] },
+                        "disk" : VIRTIO_DISK,
+                        "net"  : VIRTIO_NET,
+                        "input": USB_TABLET,
+                     }},
+        "fedora12": { "label": "Fedora 12", "distro": "fedora",
+                      "devices" : {
+                        "disk" : VIRTIO_DISK,
+                        "net"  : VIRTIO_NET,
+                        "input": USB_TABLET,
+                     }},
+        "fedora13": { "label": "Fedora 13", "distro": "fedora",
+                      "devices" : {
+                        "disk" : VIRTIO_DISK,
+                        "net"  : VIRTIO_NET,
+                        "input": USB_TABLET,
                      }},
         "sles10": { "label": "Suse Linux Enterprise Server",
                     "distro": "suse" },
+        "sles11": { "label": "Suse Linux Enterprise Server 11",
+                    "distro": "suse",
+                      "devices" : {
+                        "disk" : VIRTIO_DISK,
+                        "net"  : VIRTIO_NET,
+                      },
+                  },
         "debianetch": { "label": "Debian Etch", "distro": "debian" },
         "debianlenny": { "label": "Debian Lenny", "distro": "debian",
                       "devices" : {
-                        "disk" : { "bus"   : [ (["kvm"], "virtio") ] },
-                        "net"  : { "model" : [ (["kvm"], "virtio") ] }
+                        "disk" : VIRTIO_DISK,
+                        "net"  : VIRTIO_NET,
                       }},
+        "debiansqueeze": { "label": "Debian Squeeze", "distro": "debian",
+                      "devices" : {
+                        "disk" : VIRTIO_DISK,
+                        "net"  : VIRTIO_NET,
+                        "input": USB_TABLET,
+                     }},
         "ubuntuhardy": { "label": "Ubuntu 8.04 LTS (Hardy Heron)",
                          "distro": "ubuntu",
                          "devices" : {
-                            "net"  : { "model" : [ (["kvm"], "virtio") ] }
+                            "net"  : VIRTIO_NET,
                          }},
         "ubuntuintrepid": { "label": "Ubuntu 8.10 (Intrepid Ibex)",
                             "distro": "ubuntu",
                             "devices" : {
-                                "net"  : { "model" : [ (["kvm"], "virtio") ] }
+                              "net"  : VIRTIO_NET,
                            }},
         "ubuntujaunty": { "label": "Ubuntu 9.04 (Jaunty Jackalope)",
                           "distro": "ubuntu",
                           "devices" : {
-                            "net"  : { "model" : [ (["kvm"], "virtio") ] },
-                            "disk" : { "bus"   : [ (["kvm"], "virtio") ] }
+                            "disk" : VIRTIO_DISK,
+                            "net"  : VIRTIO_NET,
+                        }},
+        "ubuntukarmic": { "label": "Ubuntu 9.10 (Karmic Koala)",
+                          "distro": "ubuntu",
+                          "devices" : {
+                            "disk" : VIRTIO_DISK,
+                            "net"  : VIRTIO_NET,
                         }},
         "generic24": { "label": "Generic 2.4.x kernel" },
         "generic26": { "label": "Generic 2.6.x kernel" },
         "virtio26": { "sortby": "genericvirtio26",
                       "label": "Generic 2.6.25 or later kernel with virtio",
                       "devices" : {
-                        "disk" : { "bus"   : [ (["kvm"], "virtio") ] },
-                        "net"  : { "model" : [ (["kvm"], "virtio") ] }
+                            "disk" : VIRTIO_DISK,
+                            "net"  : VIRTIO_NET,
                     }},
 
     },
 },
 
-"windows": { \
+"windows": {
     "label": "Windows",
     "clock": "localtime",
     "continue": True,
     "devices" : {
-        "input" : { "type" : [ (["all"], "tablet") ],
-                    "bus"  : [ (["all"], "usb"), ] },
+        "input" : USB_TABLET,
     },
-    "variants": { \
+    "variants": {
         "winxp":{ "label": "Microsoft Windows XP (x86)",
-                  "acpi": False, "apic": False },
+                  "acpi": [ ("xen", 3001000, False),
+                            ("all", True ), ],
+                  "apic": [ ("xen", 3001000, False),
+                            ("all", True ), ], },
         "winxp64":{ "label": "Microsoft Windows XP (x86_64)" },
         "win2k": { "label": "Microsoft Windows 2000",
-                   "acpi": False, "apic": False },
+                  "acpi": [ ("xen", 3001000, False),
+                            ("all", True ), ],
+                  "apic": [ ("xen", 3001000, False),
+                            ("all", True ), ], },
         "win2k3": { "label": "Microsoft Windows 2003" },
         "win2k8": { "label": "Microsoft Windows 2008" },
         "vista": { "label": "Microsoft Windows Vista" },
+        "win7": { "label": "Microsoft Windows 7" }
     },
 },
 
@@ -170,23 +325,21 @@ OS_TYPES = {\
     "variants": {
         "solaris9": { "label": "Sun Solaris 9", },
         "solaris10": { "label": "Sun Solaris 10",
-                       "devices" : { "input" : {
-                         "type" : [ (["all"], "tablet") ],
-                         "bus"  : [ (["all"], "usb"), ]
-                         } },
+                       "devices" : {
+                        "input" : USB_TABLET,
+                         },
                        },
         "opensolaris": { "label": "Sun OpenSolaris",
-                       "devices" : { "input" : {
-                           "type" : [ (["all"], "tablet") ],
-                           "bus"  : [ (["all"], "usb"), ]
-                         } },
+                       "devices" : {
+                           "input" : USB_TABLET,
+                         },
                        },
     },
 },
 
 "unix": {
     "label": "UNIX",
-    "variants": { \
+    "variants": {
         "freebsd6": { "label": "Free BSD 6.x" ,
                       # http://www.nabble.com/Re%3A-Qemu%3A-bridging-on-FreeBSD-7.0-STABLE-p15919603.html
                       "devices" : {
@@ -205,13 +358,13 @@ OS_TYPES = {\
     },
 },
 
-"other": { \
+"other": {
     "label": "Other",
-    "variants": { \
+    "variants": {
         "msdos": { "label": "MS-DOS", "acpi": False, "apic": False },
         "netware4": { "label": "Novell Netware 4" },
         "netware5": { "label": "Novell Netware 5" },
-        "netware6": { "label": "Novell Netware 6" },
+        "netware6": { "label": "Novell Netware 6", "pv_cdrom_install": True, },
         "generic": { "label": "Generic" },
     },
 },}
