@@ -38,11 +38,14 @@ from sys import stderr
 
 import libvirt
 from virtinst import _virtinst as _
+import virtinst
 import CapabilitiesParser
-from User import User
+import User
+import support
 
 KEYBOARD_DIR = "/etc/sysconfig/keyboard"
 XORG_CONF = "/etc/X11/xorg.conf"
+CONSOLE_SETUP_CONF = "/etc/default/console-setup"
 
 def default_route(nic = None):
     if platform.system() == 'SunOS':
@@ -75,59 +78,24 @@ def default_route(nic = None):
             continue
     return None
 
-def _default_nic():
-    """Return the default NIC to use, if one is specified.
-       This is NOT part of the API and may change at will."""
-
-    dev = ''
-
-    if platform.system() != 'SunOS':
-        return dev
-
-    # XXX: fails without PRIV_XVM_CONTROL
-    proc = subprocess.Popen(['/usr/lib/xen/bin/xenstore-read',
-        'device-misc/vif/default-nic'], stdout=subprocess.PIPE,
-        stderr=subprocess.PIPE)
-    out = proc.stdout.readlines()
-    if len(out) > 0:
-        dev = out[0].rstrip()
-
-    return dev
 
 def default_bridge():
-    if platform.system() == 'SunOS':
-        return _default_nic()
-
-    rt = default_route()
-    if rt is None:
-        defn = None
+    ret = virtinst._util.default_bridge2(None)
+    if not ret:
+        # Maintain this behavior for back compat
+        ret = "xenbr0"
     else:
-        defn = int(rt[-1])
+        ret = ret[1]
 
-    if defn is None:
-        return "xenbr0"
-    else:
-        return "xenbr%d"%(defn)
+    return ret
 
 def default_network(conn):
-    if platform.system() == 'SunOS':
-        return ["bridge", _default_nic()]
+    ret = virtinst._util.default_bridge2(conn)
+    if not ret:
+        # FIXME: Check that this exists
+        ret = ["network", "default"]
 
-    dev = default_route()
-
-    if dev is not None and not is_uri_remote(conn.getURI()):
-        # New style peth0 == phys dev, eth0 == bridge, eth0 == default route
-        if os.path.exists("/sys/class/net/%s/bridge" % dev):
-            return ["bridge", dev]
-
-        # Old style, peth0 == phys dev, eth0 == netloop, xenbr0 == bridge,
-        # vif0.0 == netloop enslaved, eth0 == default route
-        defn = int(dev[-1])
-        if os.path.exists("/sys/class/net/peth%d/brport" % defn) and \
-           os.path.exists("/sys/class/net/xenbr%d/bridge" % defn):
-            return ["bridge", "xenbr%d" % defn]
-
-    return ["network", "default"]
+    return ret
 
 def default_connection():
     if os.path.exists('/var/lib/xend'):
@@ -140,7 +108,7 @@ def default_connection():
         os.path.exists("/usr/bin/qemu-kvm") or \
         os.path.exists("/usr/bin/kvm") or \
         os.path.exists("/usr/bin/xenner"):
-        if User.current().has_priv(User.PRIV_QEMU_SYSTEM):
+        if User.User.current().has_priv(User.User.PRIV_QEMU_SYSTEM):
             return "qemu:///system"
         else:
             return "qemu:///session"
@@ -214,7 +182,7 @@ def randomMAC(type = "xen"):
     """Generate a random MAC address.
 
     00-16-3E allocated to xensource
-    54-52-00 used by qemu/kvm
+    52-54-00 used by qemu/kvm
 
     The OUI list is available at http://standards.ieee.org/regauth/oui/oui.txt.
 
@@ -227,12 +195,12 @@ def randomMAC(type = "xen"):
     True
     >>> randomMAC("xen").startswith("00:16:36")
     True
-    >>> randomMAC("qemu").startswith("54:52:00")
+    >>> randomMAC("qemu").startswith("52:54:00")
     True
 
     @return: MAC address string
     """
-    ouis = { 'xen': [ 0x00, 0x16, 0x36 ], 'qemu': [ 0x54, 0x52, 0x00 ] }
+    ouis = { 'xen': [ 0x00, 0x16, 0x36 ], 'qemu': [ 0x52, 0x54, 0x00 ] }
 
     try:
         oui = ouis[type]
@@ -354,12 +322,32 @@ def _xorg_keymap():
         f.close()
     return kt
 
+def _console_setup_keymap():
+    """Look in /etc/default/console-setup for the host machine's keymap, and attempt to
+       map it to a keymap supported by qemu"""
+
+    kt = None
+    try:
+        f = open(CONSOLE_SETUP_CONF, "r")
+    except IOError, e:
+        logging.debug('Could not open "%s": %s ' % (CONSOLE_SETUP_CONF, str(e)))
+    else:
+        keymap_re = re.compile(r'\s*XKBLAYOUT="(?P<kt>[a-z-]+)"')
+        for line in f:
+            m = keymap_re.match(line)
+            if m:
+                kt = m.group('kt')
+                break
+        else:
+            logging.debug("Didn't find keymap in '%s'!" % XORG_CONF)
+        f.close()
+    return kt
+
 def default_keymap():
     """Look in /etc/sysconfig for the host machine's keymap, and attempt to
        map it to a keymap supported by qemu"""
 
     # Set keymap to same as hosts
-    import keytable
     default = "en-us"
     keymap = None
 
@@ -369,6 +357,8 @@ def default_keymap():
     except IOError, e:
         logging.debug('Could not open "/etc/sysconfig/keyboard" ' + str(e))
         kt = _xorg_keymap()
+        if not kt:
+            kt = _console_setup_keymap()
     else:
         while 1:
             s = f.readline()
@@ -392,26 +382,7 @@ def default_keymap():
 
     kt = kt.lower()
 
-    # Try a simple lookup in the keytable
-    if keytable.keytable.has_key(kt.lower()):
-        keymap = keytable.keytable[kt]
-    else:
-        # Try a more intelligent lookup: strip out all '-' and '_', sort
-        # the keytable keys putting the longest first, then compare
-        # by string prefix
-        def len_cmp(a, b):
-            return len(b) - len(a)
-
-        clean_kt = kt.replace("-", "").replace("_", "")
-        sorted_keys = sorted(keytable.keytable.keys(), len_cmp)
-
-        for key in sorted_keys:
-            origkey = key
-            key = key.replace("-", "").replace("_","")
-
-            if clean_kt.startswith(key):
-                keymap = keytable.keytable[origkey]
-                break
+    keymap = check_keytable(kt)
 
     if not keymap:
         logging.debug("Didn't match keymap '%s' in keytable!" % kt)
@@ -525,19 +496,7 @@ def get_uri_driver(uri):
 
 def is_storage_capable(conn):
     """check if virConnectPtr passed has storage API support"""
-    if not conn:
-        return False
-    if not isinstance(conn, libvirt.virConnect):
-        raise ValueError(_("'conn' must be a virConnect instance."))
-    try:
-        if not dir(conn).count("listStoragePools"):
-            return False
-        conn.listStoragePools()
-    except libvirt.libvirtError, e:
-        if e.get_error_code() == libvirt.VIR_ERR_RPC or \
-           e.get_error_code() == libvirt.VIR_ERR_NO_SUPPORT:
-            return False
-    return True
+    return support.check_conn_support(conn, support.SUPPORT_CONN_STORAGE)
 
 def get_xml_path(xml, path=None, func=None):
     """
@@ -556,7 +515,7 @@ def get_xml_path(xml, path=None, func=None):
             ret = ctx.xpathEval(path)
             if ret != None:
                 if type(ret) == list:
-                    if len(ret) == 1:
+                    if len(ret) >= 1:
                         result = ret[0].content
                 else:
                     result = ret
@@ -599,6 +558,31 @@ def lookup_pool_by_path(conn, path):
                 return p
     return None
 
+def check_keytable(kt):
+    import keytable
+    keymap = None
+    # Try a simple lookup in the keytable
+    if keytable.keytable.has_key(kt.lower()):
+        return keytable.keytable[kt]
+    else:
+        # Try a more intelligent lookup: strip out all '-' and '_', sort
+        # the keytable keys putting the longest first, then compare
+        # by string prefix
+        def len_cmp(a, b):
+            return len(b) - len(a)
+
+        clean_kt = kt.replace("-", "").replace("_", "")
+        sorted_keys = sorted(keytable.keytable.keys(), len_cmp)
+
+        for key in sorted_keys:
+            origkey = key
+            key = key.replace("-", "").replace("_","")
+
+            if clean_kt.startswith(key):
+                return keytable.keytable[origkey]
+
+    return keymap
+
 def _test():
     import doctest
     doctest.testmod()
